@@ -6,8 +6,14 @@ from urllib.parse import urlparse
 
 import click
 
+from launchable.utils.launchable_client import LaunchableClient
+from launchable.utils.tracking import Tracking, TrackingClient
+
+from ...app import Application
+from ...utils.commands import Command
 from ...utils.commit_ingester import upload_commits
-from ...utils.env_keys import REPORT_ERROR_KEY
+from ...utils.env_keys import COMMIT_TIMEOUT, REPORT_ERROR_KEY
+from ...utils.fail_fast_mode import set_fail_fast_mode, warn_and_exit_if_fail_fast_mode
 from ...utils.git_log_parser import parse_git_log
 from ...utils.http_client import get_base_url
 from ...utils.java import cygpath, get_java_command
@@ -49,26 +55,43 @@ def commit(ctx, source: str, executable: bool, max_days: int, scrub_pii: bool, i
     if executable == 'docker':
         sys.exit("--executable docker is no longer supported")
 
+    tracking_client = TrackingClient(Command.COMMIT, app=ctx.obj)
+    client = LaunchableClient(tracking_client=tracking_client, app=ctx.obj)
+    set_fail_fast_mode(client.is_fail_fast_mode())
+
     if import_git_log_output:
-        _import_git_log(import_git_log_output, ctx.obj.dry_run)
+        _import_git_log(import_git_log_output, ctx.obj)
         return
 
+    # Commit messages are not collected in the default.
+    is_collect_message = False
+    is_collect_files = False
     try:
-        exec_jar(os.path.abspath(source), max_days, ctx.obj.dry_run)
+        res = client.request("get", "commits/collect/options")
+        res.raise_for_status()
+        is_collect_message = res.json().get("commitMessage", False)
+        is_collect_files = res.json().get("files", False)
+    except Exception as e:
+        tracking_client.send_error_event(
+            event_name=Tracking.ErrorEvent.INTERNAL_CLI_ERROR,
+            stack_trace=str(e),
+            api="commits/options",
+        )
+        client.print_exception_and_recover(e)
+
+    cwd = os.path.abspath(source)
+    try:
+        exec_jar(cwd, max_days, ctx.obj, is_collect_message, is_collect_files)
     except Exception as e:
         if os.getenv(REPORT_ERROR_KEY):
             raise e
         else:
-            click.echo(click.style(
-                "Can't get commit history from `{}`. Do you run command root of git-controlled directory? "
-                "If not, please set a directory use by --source option."
-                .format(os.path.abspath(source)),
-                fg='yellow'),
-                err=True)
-            print(e)
+            warn_and_exit_if_fail_fast_mode(
+                "Couldn't get commit history from `{}`. Do you run command root of git-controlled directory? "
+                "If not, please set a directory use by --source option.\nerror: {}".format(cwd, e))
 
 
-def exec_jar(source: str, max_days: int, dry_run: bool):
+def exec_jar(source: str, max_days: int, app: Application, is_collect_message: bool, is_collect_files: bool):
     java = get_java_command()
 
     if not java:
@@ -88,13 +111,21 @@ def exec_jar(source: str, max_days: int, dry_run: bool):
         "{}/intake/".format(base_url),
         "-max-days",
         str(max_days),
-        "-scrub-pii",
+        "-scrub-pii"
     ])
 
     if Logger().logger.isEnabledFor(LOG_LEVEL_AUDIT):
         command.append("-audit")
-    if dry_run:
+    if app.dry_run:
         command.append("-dry-run")
+    if app.skip_cert_verification:
+        command.append("-skip-cert-verification")
+    if is_collect_message:
+        command.append("-commit-message")
+    if is_collect_files:
+        command.append("-files")
+    if os.getenv(COMMIT_TIMEOUT):
+        command.append("-enable-timeout")
     command.append(cygpath(source))
 
     subprocess.run(
@@ -104,19 +135,16 @@ def exec_jar(source: str, max_days: int, dry_run: bool):
     )
 
 
-def _import_git_log(output_file: str, dry_run: bool):
+def _import_git_log(output_file: str, app: Application):
     try:
         with click.open_file(output_file) as fp:
             commits = parse_git_log(fp)
-        upload_commits(commits, dry_run)
+        upload_commits(commits, app)
     except Exception as e:
         if os.getenv(REPORT_ERROR_KEY):
             raise e
         else:
-            click.echo(
-                click.style("Failed to import the git-log output", fg='yellow'),
-                err=True)
-            print(e)
+            warn_and_exit_if_fail_fast_mode("Failed to import the git-log output\n error: {}".format(e))
 
 
 def _build_proxy_option(https_proxy: Optional[str]) -> List[str]:
