@@ -1,19 +1,22 @@
+import datetime
 import os
 import re
 import sys
 from http import HTTPStatus
-from typing import List, Optional
+from typing import Optional, Sequence, Tuple
 
 import click
 
-from launchable.utils.key_value_type import normalize_key_value_types
+from launchable.utils.click import DATETIME_WITH_TZ, validate_past_datetime
 from launchable.utils.link import LinkKind, capture_link
+from launchable.utils.tracking import Tracking, TrackingClient
 
-from ...utils.click import KeyValueType
-from ...utils.env_keys import REPORT_ERROR_KEY
-from ...utils.http_client import LaunchableClient
+from ...utils.click import KEY_VALUE
+from ...utils.commands import Command
+from ...utils.fail_fast_mode import FailFastModeValidateParams, fail_fast_mode_validate, set_fail_fast_mode
+from ...utils.launchable_client import LaunchableClient
 from ...utils.no_build import NO_BUILD_BUILD_NAME
-from ...utils.session import read_build, write_session
+from ...utils.session import _session_file_path, read_build, write_session
 
 LAUNCHABLE_SESSION_DIR_KEY = 'LAUNCHABLE_SESSION_DIR'
 
@@ -50,7 +53,8 @@ def _validate_session_name(ctx, param, value):
     "flavor",
     help='flavors',
     metavar='KEY=VALUE',
-    cls=KeyValueType,
+    type=KEY_VALUE,
+    default=(),
     multiple=True,
 )
 @click.option(
@@ -64,37 +68,62 @@ def _validate_session_name(ctx, param, value):
     'links',
     help="Set external link of title and url",
     multiple=True,
-    default=[],
-    cls=KeyValueType,
+    default=(),
+    type=KEY_VALUE,
 )
 @click.option(
     "--no-build",
     "is_no_build",
     help="If you want to only send test reports, please use this option",
     is_flag=True,
-    hidden=True,
 )
 @click.option(
     '--session-name',
     'session_name',
     help='test session name',
     required=False,
-    hidden=True,
     type=str,
     metavar='SESSION_NAME',
     callback=_validate_session_name,
 )
+@click.option(
+    '--lineage',
+    'lineage',
+    help='Set lineage name. A lineage is a set of test sessions grouped and this option value will be used for a lineage name.',
+    required=False,
+    type=str,
+    metavar='LINEAGE',
+)
+@click.option(
+    '--test-suite',
+    'test_suite',
+    help='Set test suite name. A test suite is a collection of test sessions. Setting a test suite allows you to manage data over test sessions and lineages.',  # noqa: E501
+    required=False,
+    type=str,
+    metavar='TEST_SUITE',
+)
+@click.option(
+    '--timestamp',
+    'timestamp',
+    help='Used to overwrite the session time when importing historical data. Note: Format must be `YYYY-MM-DDThh:mm:ssTZD` or `YYYY-MM-DDThh:mm:ss` (local timezone applied)',  # noqa: E501
+    type=DATETIME_WITH_TZ,
+    default=None,
+    callback=validate_past_datetime,
+)
 @click.pass_context
 def session(
     ctx: click.core.Context,
     build_name: str,
     save_session_file: bool,
     print_session: bool = True,
-    flavor: List[str] = [],
+    flavor: Sequence[Tuple[str, str]] = [],
     is_observation: bool = False,
-    links: List[str] = [],
+    links: Sequence[Tuple[str, str]] = [],
     is_no_build: bool = False,
     session_name: Optional[str] = None,
+    lineage: Optional[str] = None,
+    test_suite: Optional[str] = None,
+    timestamp: Optional[datetime.datetime] = None,
 ):
     """
     print_session is for backward compatibility.
@@ -105,6 +134,17 @@ def session(
     you should set print_session = False because users don't expect to print session ID to the subset output.
     """
 
+    tracking_client = TrackingClient(Command.RECORD_SESSION, app=ctx.obj)
+    client = LaunchableClient(app=ctx.obj, tracking_client=tracking_client)
+    set_fail_fast_mode(client.is_fail_fast_mode())
+
+    fail_fast_mode_validate(FailFastModeValidateParams(
+        command=Command.RECORD_SESSION,
+        build=build_name,
+        is_no_build=is_no_build,
+        test_suite=test_suite,
+    ))
+
     if not is_no_build and not build_name:
         raise click.UsageError("Error: Missing option '--build'")
 
@@ -112,38 +152,51 @@ def session(
         build = read_build()
         if build and build != "":
             raise click.UsageError(
-                'The cli already created `.launchable file`. If you want to use `--no-build option`, please remove `.launchable` file before executing.')  # noqa: E501
+                "The cli already created '{}'. If you want to use the '--no-build' option, please remove this file first.".format(_session_file_path()))  # noqa: E501
 
         build_name = NO_BUILD_BUILD_NAME
 
-    client = LaunchableClient(dry_run=ctx.obj.dry_run)
-
     if session_name:
         sub_path = "builds/{}/test_session_names/{}".format(build_name, session_name)
-        res = client.request("get", sub_path)
+        try:
+            res = client.request("get", sub_path)
 
-        if res.status_code != 404:
-            raise click.UsageError(
-                'This session name ({}) is already used. Please set another name.'.format(session_name))
+            if res.status_code != 404:
+                msg = "This session name ({}) is already used. Please set another name.".format(session_name)
+                click.echo(click.style(
+                    msg,
+                    fg='red'),
+                    err=True)
+                tracking_client.send_error_event(
+                    event_name=Tracking.ErrorEvent.USER_ERROR,
+                    stack_trace=msg,
+                )
+                sys.exit(2)
+        except Exception as e:
+            tracking_client.send_error_event(
+                event_name=Tracking.ErrorEvent.INTERNAL_CLI_ERROR,
+                stack_trace=str(e),
+            )
+            client.print_exception_and_recover(e)
 
-    flavor_dict = {}
-    for f in normalize_key_value_types(flavor):
-        flavor_dict[f[0]] = f[1]
+    flavor_dict = dict(flavor)
 
     payload = {
         "flavors": flavor_dict,
         "isObservation": is_observation,
         "noBuild": is_no_build,
+        "lineage": lineage,
+        "testSuite": test_suite,
+        "timestamp": timestamp.isoformat() if timestamp else None,
     }
 
     _links = capture_link(os.environ)
-    if len(links) != 0:
-        for link in normalize_key_value_types(links):
-            _links.append({
-                "title": link[0],
-                "url": link[1],
-                "kind": LinkKind.CUSTOM_LINK.name,
-            })
+    for link in links:
+        _links.append({
+            "title": link[0],
+            "url": link[1],
+            "kind": LinkKind.CUSTOM_LINK.name,
+        })
     payload["links"] = _links
 
     try:
@@ -151,12 +204,16 @@ def session(
         res = client.request("post", sub_path, payload=payload)
 
         if res.status_code == HTTPStatus.NOT_FOUND:
+            msg = "Build {} was not found." \
+                "Make sure to run `launchable record build --name {}` before you run this command.".format(
+                    build_name, build_name)
+            tracking_client.send_error_event(
+                event_name=Tracking.ErrorEvent.INTERNAL_CLI_ERROR,
+                stack_trace=msg,
+            )
             click.echo(
                 click.style(
-                    "Build {} was not found. "
-                    "Make sure to run `launchable record build --name {}` before you run this command.".format(
-                        build_name,
-                        build_name),
+                    msg,
                     'yellow'),
                 err=True,
             )
@@ -177,10 +234,11 @@ def session(
             click.echo("{}/{}".format(sub_path, session_id), nl=False)
 
     except Exception as e:
-        if os.getenv(REPORT_ERROR_KEY):
-            raise e
-        else:
-            click.echo(e, err=True)
+        tracking_client.send_error_event(
+            event_name=Tracking.ErrorEvent.INTERNAL_CLI_ERROR,
+            stack_trace=str(e),
+        )
+        client.print_exception_and_recover(e)
 
     if session_name:
         try:
@@ -191,10 +249,11 @@ def session(
                 session_name=session_name,
             )
         except Exception as e:
-            if os.getenv(REPORT_ERROR_KEY):
-                raise e
-            else:
-                click.echo(e, err=True)
+            tracking_client.send_error_event(
+                event_name=Tracking.ErrorEvent.INTERNAL_CLI_ERROR,
+                stack_trace=str(e),
+            )
+            client.print_exception_and_recover(e)
 
 
 def add_session_name(
