@@ -2,10 +2,9 @@ import datetime
 import glob
 import os
 import re
-import traceback
 import xml.etree.ElementTree as ET
 from http import HTTPStatus
-from typing import Callable, Dict, Generator, List, Optional, Tuple, Union
+from typing import Callable, Dict, Generator, List, Optional, Sequence, Tuple, Union
 
 import click
 from dateutil.parser import parse
@@ -14,16 +13,19 @@ from more_itertools import ichunked
 from tabulate import tabulate
 
 from launchable.utils.authentication import ensure_org_workspace
+from launchable.utils.tracking import Tracking, TrackingClient
 
 from ...testpath import FilePathNormalizer, TestPathComponent, unparse_test_path
-from ...utils.click import KeyValueType
-from ...utils.env_keys import REPORT_ERROR_KEY
+from ...utils.click import DATETIME_WITH_TZ, KEY_VALUE, validate_past_datetime
+from ...utils.commands import Command
 from ...utils.exceptions import InvalidJUnitXMLException
-from ...utils.http_client import LaunchableClient
+from ...utils.fail_fast_mode import (FailFastModeValidateParams, fail_fast_mode_validate,
+                                     set_fail_fast_mode, warn_and_exit_if_fail_fast_mode)
+from ...utils.launchable_client import LaunchableClient
 from ...utils.logger import Logger
 from ...utils.no_build import NO_BUILD_BUILD_NAME, NO_BUILD_TEST_SESSION_ID
 from ...utils.session import parse_session, read_build
-from ..helper import find_or_create_session
+from ..helper import find_or_create_session, time_ns
 from .case_event import CaseEvent, CaseEventType
 
 GROUP_NAME_RULE = re.compile("^[a-zA-Z0-9][a-zA-Z0-9_-]*$")
@@ -82,7 +84,8 @@ def _validate_group(ctx, param, value):
     "flavor",
     help='flavors',
     metavar='KEY=VALUE',
-    cls=KeyValueType,
+    type=KEY_VALUE,
+    default=(),
     multiple=True,
 )
 @click.option(
@@ -126,25 +129,47 @@ def _validate_group(ctx, param, value):
     'links',
     help="Set external link of title and url",
     multiple=True,
-    default=[],
-    cls=KeyValueType,
+    default=(),
+    type=KEY_VALUE,
 )
 @click.option(
     '--no-build',
     'is_no_build',
     help="If you want to only send test reports, please use this option",
     is_flag=True,
-    hidden=True,
 )
 @click.option(
     '--session-name',
     'session_name',
     help='test session name',
     required=False,
-    hidden=True,
     type=str,
     metavar='SESSION_NAME',
 )
+@click.option(
+    '--lineage',
+    'lineage',
+    help='Set lineage name. This option value will be passed to the record session command if a session isn\'t created yet.',
+    required=False,
+    type=str,
+    metavar='LINEAGE',
+)
+@click.option(
+    '--test-suite',
+    'test_suite',
+    help='Set test suite name. This option value will be passed to the record session command if a session isn\'t created yet.',  # noqa: E501
+    required=False,
+    type=str,
+    metavar='TEST_SUITE',
+)
+@click.option(
+    '--timestamp',
+    'timestamp',
+    help='Used to overwrite the test executed times when importing historical data. Note: Format must be `YYYY-MM-DDThh:mm:ssTZD` or `YYYY-MM-DDThh:mm:ss` (local timezone applied)',  # noqa: E501
+    type=DATETIME_WITH_TZ,
+    default=None,
+    callback=validate_past_datetime,
+)
 @click.pass_context
 def tests(
     context: click.core.Context,
@@ -153,14 +178,17 @@ def tests(
     build_name: Optional[str],
     post_chunk: int,
     subsetting_id: str,
-    flavor,
+    flavor: Sequence[Tuple[str, str]],
     no_base_path_inference: bool,
     report_paths: bool,
     group: str,
     is_allow_test_before_build: bool,
-    links: List[str] = [],
+    links: Sequence[Tuple[str, str]] = (),
     is_no_build: bool = False,
-    session_name: Optional[str] = None
+    session_name: Optional[str] = None,
+    lineage: Optional[str] = None,
+    test_suite: Optional[str] = None,
+    timestamp: Optional[datetime.datetime] = None,
 ):
     logger = Logger()
 
@@ -168,13 +196,38 @@ def tests(
 
     test_runner = context.invoked_subcommand
 
-    client = LaunchableClient(test_runner=test_runner, dry_run=context.obj.dry_run)
+    tracking_client = TrackingClient(Command.RECORD_TESTS, app=context.obj)
+    client = LaunchableClient(test_runner=test_runner, app=context.obj, tracking_client=tracking_client)
+    set_fail_fast_mode(client.is_fail_fast_mode())
+
+    fail_fast_mode_validate(FailFastModeValidateParams(
+        command=Command.RECORD_TESTS,
+        session=session,
+        build=build_name,
+        flavor=flavor,
+        links=links,
+        is_no_build=is_no_build,
+        test_suite=test_suite,
+    ))
 
     file_path_normalizer = FilePathNormalizer(base_path, no_base_path_inference=no_base_path_inference)
 
     if is_no_build and (read_build() and read_build() != ""):
-        raise click.UsageError(
-            'The cli already created `.launchable` file. If you want to use `--no-build` option, please remove `.launchable` file before executing.')  # noqa: E501
+        msg = 'The cli already created `.launchable` file.' \
+            'If you want to use `--no-build` option, please remove `.launchable` file before executing.'
+        tracking_client.send_error_event(
+            event_name=Tracking.ErrorEvent.INTERNAL_CLI_ERROR,
+            stack_trace=msg,
+
+        )
+        raise click.UsageError(message=msg)  # noqa: E501
+
+    if is_no_build and session:
+        warn_and_exit_if_fail_fast_mode(
+            "WARNING: `--session` and `--no-build` are set.\nUsing --session option value ({}) and ignoring `--no-build` option".format(session),  # noqa: E501
+        )
+
+        is_no_build = False
 
     try:
         if is_no_build:
@@ -187,7 +240,7 @@ def tests(
         elif session_name:
             if not build_name:
                 raise click.UsageError(
-                    '--build-name is required when you uses a --session-name option ')
+                    '--build option is required when you uses a --session-name option ')
 
             sub_path = "builds/{}/test_session_names/{}".format(build_name, session_name)
             res = client.request("get", sub_path)
@@ -202,17 +255,24 @@ def tests(
                 session=session,
                 build_name=build_name,
                 flavor=flavor,
-                links=links))
+                links=links,
+                lineage=lineage,
+                test_suite=test_suite,
+                timestamp=timestamp,
+                tracking_client=tracking_client))
             build_name = read_build()
             record_start_at = get_record_start_at(session_id, client)
 
         build_name, test_session_id = parse_session(session_id)
     except Exception as e:
-        if os.getenv(REPORT_ERROR_KEY):
-            raise e
-        else:
-            traceback.print_exc()
-            return
+        tracking_client.send_error_event(
+            event_name=Tracking.ErrorEvent.INTERNAL_CLI_ERROR,
+            stack_trace=str(e),
+        )
+        client.print_exception_and_recover(e)
+        # To prevent users from stopping the CI pipeline, the cli exits with a
+        # status code of 0, indicating that the program terminated successfully.
+        exit(0)
 
     # TODO: placed here to minimize invasion in this PR to reduce the likelihood of
     # PR merge hell. This should be moved to a top-level class
@@ -280,6 +340,18 @@ def tests(
         def is_no_build(self, is_no_build: bool):
             self._is_no_build = is_no_build
 
+        @property
+        def metadata_builder(self) -> CaseEvent.DataBuilder:
+            """
+            This function, if supplied, is used to build a test path
+            that uniquely identifies a test case
+            """
+            return self._metadata_builder
+
+        @metadata_builder.setter
+        def metadata_builder(self, v: CaseEvent.DataBuilder):
+            self._metadata_builder = v
+
         # setter only property that sits on top of the parse_func property
         def set_junitxml_parse_func(self, f: JUnitXmlParseFunc):
             """
@@ -296,11 +368,12 @@ def tests(
                 try:
                     xml = JUnitXml.fromfile(report, f)
                 except Exception as e:
-                    click.echo(click.style("Warning: error reading JUnitXml file {filename}: {error}".format(
-                        filename=report, error=e), fg="yellow"), err=True)
                     # `JUnitXml.fromfile()` will raise `JUnitXmlError` and other lxml related errors
                     # if the file has wrong format.
                     # https://github.com/weiwei/junitparser/blob/master/junitparser/junitparser.py#L321
+                    warn_and_exit_if_fail_fast_mode(
+                        "Warning: error reading JUnitXml file {filename}: {error}".format(
+                            filename=report, error=e))
                     return
                 if isinstance(xml, JUnitXml):
                     testsuites = [suite for suite in xml]
@@ -312,10 +385,11 @@ def tests(
                 try:
                     for suite in testsuites:
                         for case in suite:
-                            yield CaseEvent.from_case_and_suite(self.path_builder, case, suite, report)
+                            yield CaseEvent.from_case_and_suite(self.path_builder, case, suite, report, self.metadata_builder)
                 except Exception as e:
-                    click.echo(click.style("Warning: error parsing JUnitXml file {filename}: {error}".format(
-                        filename=report, error=e), fg="yellow"), err=True)
+                    warn_and_exit_if_fail_fast_mode(
+                        "Warning: error parsing JUnitXml file {filename}: {error}".format(
+                            filename=report, error=e))
 
             self.parse_func = parse
 
@@ -343,6 +417,7 @@ def tests(
             self.test_session_id = test_session_id
             self.session = session_id
             self.is_no_build = is_no_build
+            self.metadata_builder = CaseEvent.default_data_builder()
 
         def make_file_path_component(self, filepath) -> TestPathComponent:
             """Create a single TestPathComponent from the given file path"""
@@ -354,8 +429,13 @@ def tests(
             ctime = datetime.datetime.fromtimestamp(
                 os.path.getctime(junit_report_file))
 
-            if not self.is_allow_test_before_build and not self.is_no_build and (
-                    self.check_timestamp and ctime.timestamp() < record_start_at.timestamp()):
+            if (
+                    not self.is_allow_test_before_build  # nlqa: W503
+                    and not self.is_no_build  # noqa: W503
+                    and timestamp is None  # noqa: W503
+                    and self.check_timestamp  # noqa: W503
+                    and ctime.timestamp() < record_start_at.timestamp()  # noqa: W503
+            ):
                 format = "%Y-%m-%d %H:%M:%S"
                 logger.warning("skip: {} is too old to report. start_record_at: {} file_created_at: {}".format(
                     junit_report_file, record_start_at.strftime(format), ctime.strftime(format)))
@@ -387,6 +467,10 @@ def tests(
                             if len(tc.get('testPath', [])) == 0:
                                 continue
 
+                            # Set specific time for importing historical data
+                            if timestamp is not None:
+                                tc["createdAt"] = timestamp.isoformat()
+
                             yield tc
 
                     except Exception as e:
@@ -398,8 +482,11 @@ def tests(
                     raise Exception(exceptions)
 
             # generator that creates the payload incrementally
-            def payload(cases: Generator[TestCase, None, None],
-                        test_runner, group: str) -> Tuple[Dict[str, Union[str, List, bool]], List[Exception]]:
+            def payload(
+                    cases: Generator[TestCase, None, None],
+                    test_runner, group: str,
+                    test_suite_name: str,
+                    flavors: Dict[str, str]) -> Tuple[Dict[str, Union[str, List, dict, bool]], List[Exception]]:
                 nonlocal count
                 cs = []
                 exs = []
@@ -413,7 +500,17 @@ def tests(
                         exs.append(ex)
 
                 count += len(cs)
-                return {"events": cs, "testRunner": test_runner, "group": group, "noBuild": self.is_no_build}, exs
+                return {
+                    "events": cs,
+                    "testRunner": test_runner,
+                    "group": group,
+                    "metadata": get_env_values(client),
+                    "noBuild": self.is_no_build,
+                    # NOTE:
+                    # testSuite and flavors are applied only when the no-build option is enabled
+                    "testSuite": test_suite_name,
+                    "flavors": flavors,
+                }, exs
 
             def send(payload: Dict[str, Union[str, List]]) -> None:
                 res = client.request(
@@ -422,28 +519,18 @@ def tests(
                 if res.status_code == HTTPStatus.NOT_FOUND:
                     if session:
                         build, _ = parse_session(session)
-                        click.echo(
-                            click.style(
-                                "Session {} was not found. "
-                                "Make sure to run `launchable record session --build {}` "
-                                "before `launchable record tests`".format(
-                                    session,
-                                    build),
-                                'yellow'),
-                            err=True)
+                        warn_and_exit_if_fail_fast_mode(
+                            "Session {} was not found. Make sure to run `launchable record session --build {}` before `launchable record tests`".format(session, build))  # noqa: E501
+
                     elif build_name:
-                        click.echo(
-                            click.style(
-                                "Build {} was not found. "
-                                "Make sure to run `launchable record build --name {}` "
-                                "before `launchable record tests`".format(
-                                    build_name,
-                                    build_name),
-                                'yellow'),
-                            err=True)
+                        warn_and_exit_if_fail_fast_mode(
+                            "Build {} was not found. Make sure to run `launchable record build --name {}` before `launchable record tests`".format(build_name, build_name))  # noqa: E501
 
                 res.raise_for_status()
 
+                nonlocal is_observation
+                is_observation = res.json().get("testSession", {}).get("isObservation", False)
+
                 # If don’t override build, test session and session_id, build and test session will be made per chunk request.
                 if is_no_build:
                     self.build_name = res.json().get("build", {}).get("build", NO_BUILD_BUILD_NAME)
@@ -469,7 +556,16 @@ def tests(
                 return test_count, success_count, fail_count, duration / 60   # sec to min
 
             try:
+                start = time_ns()
                 tc = testcases(self.reports)
+                end = time_ns()
+                tracking_client.send_event(
+                    event_name=Tracking.Event.PERFORMANCE,
+                    metadata={
+                        "elapsedTime": end - start,
+                        "measurementTarget": "testcases method(parsing report file)"
+                    }
+                )
 
                 if report_paths:
                     # diagnostics mode to just report test paths
@@ -477,41 +573,50 @@ def tests(
                         print(unparse_test_path(t['testPath']))
                     return
 
+                start = time_ns()
                 exceptions = []
                 for chunk in ichunked(tc, post_chunk):
-                    p, es = payload(chunk, test_runner, group)
+                    p, es = payload(
+                        cases=chunk,
+                        test_runner=test_runner,
+                        group=group,
+                        test_suite_name=test_suite if test_suite else "",
+                        flavors=dict(flavor),
+                    )
 
                     send(p)
                     exceptions.extend(es)
-
-                res = client.request("patch", "{}/close".format(self.session),
-                                     payload={"metadata": get_env_values(client)})
-                res.raise_for_status()
-                is_observation = res.json().get("isObservation", False)
+                end = time_ns()
+                tracking_client.send_event(
+                    event_name=Tracking.Event.PERFORMANCE,
+                    metadata={
+                        "elapsedTime": end - start,
+                        "measurementTarget": "events API"
+                    }
+                )
 
                 if len(exceptions) > 0:
                     raise Exception(exceptions)
 
             except Exception as e:
-                if os.getenv(REPORT_ERROR_KEY):
-                    raise e
-                else:
-                    traceback.print_exc()
-                    return
+                tracking_client.send_error_event(
+                    event_name=Tracking.ErrorEvent.INTERNAL_CLI_ERROR,
+                    stack_trace=str(e),
+                )
+                client.print_exception_and_recover(e)
+                return
 
             if count == 0:
                 if len(self.skipped_reports) != 0:
-                    click.echo(click.style(
+                    warn_and_exit_if_fail_fast_mode(
                         "{} test report(s) were skipped because they were created before this build was recorded.\n"
-                        "Make sure to run your tests after you run `launchable record build`.".format(
-                            len(self.skipped_reports)), 'yellow'))
+                        "Make sure to run your tests after you run `launchable record build`.\n"
+                        "Otherwise, if these are really correct test reports, use the `--allow-test-before-build` option.".
+                        format(len(self.skipped_reports)))
                     return
                 else:
-                    click.echo(
-                        click.style(
-                            "Looks like tests didn't run? "
-                            "If not, make sure the right files/directories were passed into `launchable record tests`",
-                            'yellow'))
+                    warn_and_exit_if_fail_fast_mode(
+                        "Looks like tests didn't run? If not, make sure the right files/directories were passed into `launchable record tests`")  # noqa: E501
                     return
 
             file_count = len(self.reports)
@@ -536,6 +641,10 @@ def tests(
             rows = [[file_count, test_count, success_count, fail_count, duration]]
             click.echo(tabulate(rows, header, tablefmt="github", floatfmt=".2f"))
 
+            if duration == 0:
+                click.echo(click.style("\nTotal test duration is 0."
+                                       "\nPlease check whether the test duration times in report files are correct.", "yellow"))
+
             click.echo(
                 "\nVisit https://app.launchableinc.com/organizations/{organization}/workspaces/"
                 "{workspace}/test-sessions/{test_session_id} to view uploaded test results "
@@ -626,7 +735,7 @@ def get_env_values(client: LaunchableClient) -> Dict[str, str]:
     sub_path = "slack/notification/key/list"
     res = client.request("get", sub_path=sub_path)
 
-    metadata = {}  # type: Dict[str, str]
+    metadata: Dict[str, str] = {}
     if res.status_code != 200:
         return metadata
 
