@@ -5,9 +5,9 @@ import click
 
 from launchable.testpath import TestPath
 
+from ..app import Application
 from ..utils.click import FRACTION, FractionType
-from ..utils.env_keys import REPORT_ERROR_KEY
-from ..utils.http_client import LaunchableClient
+from ..utils.launchable_client import LaunchableClient
 from .test_path_writer import TestPathWriter
 
 SPLIT_BY_GROUPS_NO_GROUP_NAME = "nogroup"
@@ -97,15 +97,29 @@ def split_subset(
 
     TestPathWriter.base_path = base_path
 
-    client = LaunchableClient(test_runner=context.invoked_subcommand, dry_run=context.obj.dry_run)
+    client = LaunchableClient(test_runner=context.invoked_subcommand, app=context.obj)
 
     class SplitSubset(TestPathWriter):
-        def __init__(self, dry_run: bool = False):
-            super(SplitSubset, self).__init__(dry_run=dry_run)
+        def __init__(self, app: Application):
+            super(SplitSubset, self).__init__(app=app)
+            self.rest = rest
+            self.output_handler = self._default_output_handler
+            self.exclusion_output_handler = self._default_exclusion_output_handler
             self.split_by_groups_output_handler = self._default_split_by_groups_output_handler
             self.split_by_groups_exclusion_output_handler = self._default_split_by_groups_exclusion_output_handler
             self.is_split_by_groups_with_rest = is_split_by_groups_with_rest
             self.split_by_groups_output_dir = split_by_groups_output_dir
+            self.is_output_exclusion_rules = is_output_exclusion_rules
+
+        def _default_output_handler(self, output: List[TestPath], rests: List[TestPath]):
+            if rest:
+                self.write_file(rest, rests)
+
+            if output:
+                self.print(output)
+
+        def _default_exclusion_output_handler(self, subset: List[TestPath], rest: List[TestPath]):
+            self.output_handler(rest, subset)
 
         def _default_split_by_groups_output_handler(self, group_name: str, subset: List[TestPath], rests: List[TestPath]):
             if is_split_by_groups_with_rest:
@@ -148,8 +162,8 @@ def split_subset(
                     )
                     return
 
-            output = []
-            rests = []
+            output_subset = []
+            output_rests = []
             is_observation = False
 
             try:
@@ -230,33 +244,28 @@ def split_subset(
                 res = client.request("POST", "{}/slice".format(subset_id), payload=payload)
                 res.raise_for_status()
 
-                output = res.json().get("testPaths", [])
-                rests = res.json().get("rest", [])
+                output_subset = res.json().get("testPaths", [])
+                output_rests = res.json().get("rest", [])
                 is_observation = res.json().get("isObservation", False)
 
-                if len(output) == 0:
+                if len(output_subset) == 0:
                     click.echo(click.style(
-                        "Error: no tests found in this subset id.", 'yellow'), err=True)
+                        "Error: no tests found for this subset id.", 'yellow'), err=True)
                     return
 
                 if is_observation:
-                    output = output + rests
-                if rest:
-                    if len(rests) == 0:
-                        rests.append(output[0])
+                    output_subset = output_subset + output_rests
+                    output_rests = []
 
-                    self.write_file(rest, rests)
-
-                self.print(output)
-            except Exception as e:
-                if os.getenv(REPORT_ERROR_KEY):
-                    raise e
+                if is_output_exclusion_rules:
+                    self.exclusion_output_handler(output_subset, output_rests)
                 else:
-                    click.echo(e, err=True)
-                    click.echo(click.style(
-                        "Warning: the service failed to split subset. Falling back to running all tests", fg='yellow'),
-                        err=True)
-                    return
+                    self.output_handler(output_subset, output_rests)
+
+            except Exception as e:
+                client.print_exception_and_recover(
+                    e, "Warning: the service failed to split subset. Falling back to running all tests")
+                return
 
         def _write_split_by_groups_group_names(self, subset_group_names: List[str], rest_group_names: List[str]):
             if is_output_exclusion_rules:
@@ -304,14 +313,8 @@ def split_subset(
                     self._write_split_by_groups_group_names(subset_group_names, rest_group_names)
 
             except Exception as e:
-                if os.getenv(REPORT_ERROR_KEY):
-                    raise e
-                else:
-                    click.echo(e, err=True)
-                    click.echo(click.style(
-                        "Error: the service failed to split subset.", fg='red'),
-                        err=True)
-                    exit(1)
+                client.print_exception_and_recover(e, "Error: the service failed to split subset.", 'red')
+                exit(1)
 
         def run(self):
             if (not self._is_split_by_groups() and bin_target is None) or (self._is_split_by_groups() and bin_target):
@@ -324,4 +327,4 @@ def split_subset(
             else:
                 self.split_by_bin()
 
-    context.obj = SplitSubset(dry_run=context.obj.dry_run)
+    context.obj = SplitSubset(app=context.obj)
