@@ -1,20 +1,27 @@
 import glob
+import json
 import os
 import pathlib
 import sys
+from multiprocessing import Process
 from os.path import join
-from typing import Callable, List, Optional, TextIO, Union
+from typing import Any, Callable, Dict, List, Optional, Sequence, TextIO, Tuple, Union
 
 import click
 from tabulate import tabulate
 
 from launchable.utils.authentication import get_org_workspace
 from launchable.utils.session import parse_session
+from launchable.utils.tracking import Tracking, TrackingClient
 
+from ..app import Application
 from ..testpath import FilePathNormalizer, TestPath
-from ..utils.click import DURATION, PERCENTAGE, DurationType, KeyValueType, PercentageType
+from ..utils.click import DURATION, KEY_VALUE, PERCENTAGE, DurationType, PercentageType, ignorable_error
+from ..utils.commands import Command
 from ..utils.env_keys import REPORT_ERROR_KEY
-from ..utils.http_client import LaunchableClient
+from ..utils.fail_fast_mode import (FailFastModeValidateParams, fail_fast_mode_validate,
+                                    set_fail_fast_mode, warn_and_exit_if_fail_fast_mode)
+from ..utils.launchable_client import LaunchableClient
 from .helper import find_or_create_session
 from .test_path_writer import TestPathWriter
 
@@ -40,6 +47,12 @@ from .test_path_writer import TestPathWriter
     help='subsetting by confidence from 0% to 100%',
     type=PERCENTAGE,
 )
+@click.option(
+    '--goal-spec',
+    'goal_spec',
+    help='subsetting by programmatic goal definition',
+    type=str,
+)
 @click.option(
     '--session',
     'session',
@@ -72,7 +85,8 @@ from .test_path_writer import TestPathWriter
     "flavor",
     help='flavors',
     metavar='KEY=VALUE',
-    cls=KeyValueType,
+    type=KEY_VALUE,
+    default=(),
     multiple=True,
 )
 @click.option(
@@ -82,7 +96,8 @@ from .test_path_writer import TestPathWriter
     is_flag=True
 )
 @click.option(
-    "--no_base_path_inference",
+    "--no-base-path-inference",
+    "--no_base_path_inference",  # historical, inconsistently named
     "no_base_path_inference",
     help="""Do not guess the base path to relativize the test file paths.
 
@@ -120,10 +135,17 @@ from .test_path_writer import TestPathWriter
     help="outputs the exclude test list. Switch the subset and rest.",
     is_flag=True,
 )
+@click.option(
+    "--non-blocking",
+    "is_non_blocking",
+    help="Do not wait for subset requests in observation mode.",
+    is_flag=True,
+    hidden=True,
+)
 @click.option(
     "--ignore-flaky-tests-above",
     "ignore_flaky_tests_above",
-    help='Ignore flaky tests above the value set by this option.You can confirm flaky scores in WebApp',
+    help='Ignore flaky tests above the value set by this option. You can confirm flaky scores in WebApp',
     type=click.FloatRange(min=0, max=1.0),
 )
 @click.option(
@@ -131,15 +153,51 @@ from .test_path_writer import TestPathWriter
     'links',
     help="Set external link of title and url",
     multiple=True,
-    default=[],
-    cls=KeyValueType,
+    default=(),
+    type=KEY_VALUE,
 )
 @click.option(
     "--no-build",
     "is_no_build",
     help="If you want to only send test reports, please use this option",
     is_flag=True,
-    hidden=True,
+)
+@click.option(
+    '--session-name',
+    'session_name',
+    help='test session name',
+    required=False,
+    type=str,
+    metavar='SESSION_NAME',
+)
+@click.option(
+    '--lineage',
+    'lineage',
+    help='Set lineage name. This option value will be passed to the record session command if a session isn\'t created yet.',
+    required=False,
+    type=str,
+    metavar='LINEAGE',
+)
+@click.option(
+    "--prioritize-tests-failed-within-hours",
+    "prioritize_tests_failed_within_hours",
+    help="Prioritize tests that failed within the specified hours; maximum 720 hours (= 24 hours * 30 days)",
+    type=click.IntRange(min=0, max=24 * 30),
+)
+@click.option(
+    "--prioritized-tests-mapping",
+    "prioritized_tests_mapping_file",
+    help='Prioritize tests based on test mapping file',
+    required=False,
+    type=click.File('r'),
+)
+@click.option(
+    '--test-suite',
+    'test_suite',
+    help='Set test suite name. This option value will be passed to the record session command if a session isn\'t created yet.',  # noqa: E501
+    required=False,
+    type=str,
+    metavar='TEST_SUITE',
 )
 @click.pass_context
 def subset(
@@ -150,37 +208,156 @@ def subset(
     build_name: Optional[str],
     rest: str,
     duration: Optional[DurationType],
-    flavor: KeyValueType,
+    flavor: Sequence[Tuple[str, str]],
     confidence: Optional[PercentageType],
+    goal_spec: Optional[str],
     split: bool,
     no_base_path_inference: bool,
     ignore_new_tests: bool,
     is_observation: bool,
     is_get_tests_from_previous_sessions: bool,
     is_output_exclusion_rules: bool,
+    is_non_blocking: bool,
     ignore_flaky_tests_above: Optional[float],
-    links: List[str] = [],
+    links: Sequence[Tuple[str, str]] = (),
     is_no_build: bool = False,
+    session_name: Optional[str] = None,
+    lineage: Optional[str] = None,
+    prioritize_tests_failed_within_hours: Optional[int] = None,
+    prioritized_tests_mapping_file: Optional[TextIO] = None,
+    test_suite: Optional[str] = None,
 ):
+    app = context.obj
+    tracking_client = TrackingClient(Command.SUBSET, app=app)
+    client = LaunchableClient(
+        test_runner=context.invoked_subcommand,
+        app=app,
+        tracking_client=tracking_client)
 
-    if is_observation and is_get_tests_from_previous_sessions:
-        click.echo(
-            click.style(
-                "Cannot use --observation and --get-tests-from-previous-sessions options at the same time",
-                fg="red"),
-            err=True,
-        )
-        sys.exit(1)
-
-    session_id = find_or_create_session(
-        context=context,
+    set_fail_fast_mode(client.is_fail_fast_mode())
+    fail_fast_mode_validate(FailFastModeValidateParams(
+        command=Command.SUBSET,
         session=session,
-        build_name=build_name,
+        build=build_name,
         flavor=flavor,
         is_observation=is_observation,
         links=links,
         is_no_build=is_no_build,
-    )
+        test_suite=test_suite,
+    ))
+
+    if is_observation and is_output_exclusion_rules:
+        msg = (
+            "WARNING: --observation and --output-exclusion-rules are set. "
+            "No output will be generated."
+        )
+        click.echo(
+            click.style(
+                msg,
+                fg="yellow"),
+            err=True,
+        )
+        tracking_client.send_error_event(
+            event_name=Tracking.ErrorEvent.WARNING_ERROR,
+            stack_trace=msg
+        )
+
+    if prioritize_tests_failed_within_hours is not None and prioritize_tests_failed_within_hours > 0:
+        if ignore_new_tests or (ignore_flaky_tests_above is not None and ignore_flaky_tests_above > 0):
+            msg = (
+                "Cannot use --ignore-new-tests or --ignore-flaky-tests-above options "
+                "with --prioritize-tests-failed-within-hours"
+            )
+            click.echo(
+                click.style(
+                    msg,
+                    fg="red"),
+                err=True,
+            )
+            tracking_client.send_error_event(
+                event_name=Tracking.ErrorEvent.INTERNAL_CLI_ERROR,
+                stack_trace=msg,
+            )
+            sys.exit(1)
+
+    if is_no_build and session:
+        warn_and_exit_if_fail_fast_mode(
+            "WARNING: `--session` and `--no-build` are set.\nUsing --session option value ({}) and ignoring `--no-build` option".format(session))  # noqa: E501
+        is_no_build = False
+
+    session_id = None
+
+    try:
+        if session_name:
+            if not build_name:
+                raise click.UsageError(
+                    '--build option is required when you use a --session-name option ')
+            sub_path = "builds/{}/test_session_names/{}".format(build_name, session_name)
+            client = LaunchableClient(test_runner=context.invoked_subcommand, app=context.obj, tracking_client=tracking_client)
+            res = client.request("get", sub_path)
+            res.raise_for_status()
+            session_id = "builds/{}/test_sessions/{}".format(build_name, res.json().get("id"))
+        else:
+            session_id = find_or_create_session(
+                context=context,
+                session=session,
+                build_name=build_name,
+                flavor=flavor,
+                is_observation=is_observation,
+                links=links,
+                is_no_build=is_no_build,
+                lineage=lineage,
+                tracking_client=tracking_client,
+                test_suite=test_suite,
+            )
+    except click.UsageError as e:
+        click.echo(
+            click.style(
+                str(e),
+                fg="red"),
+            err=True,
+        )
+        tracking_client.send_error_event(event_name=Tracking.ErrorEvent.USER_ERROR, stack_trace=str(e))
+        sys.exit(1)
+    except Exception as e:
+        tracking_client.send_error_event(
+            event_name=Tracking.ErrorEvent.INTERNAL_CLI_ERROR,
+            stack_trace=str(e),
+
+        )
+        if os.getenv(REPORT_ERROR_KEY):
+            raise e
+        else:
+            click.echo(ignorable_error(e), err=True)
+
+    if is_non_blocking:
+        if (not is_observation) and session_id:
+            try:
+                client = LaunchableClient(
+                    app=app,
+                    tracking_client=tracking_client)
+                res = client.request("get", session_id)
+                is_observation_in_recorded_session = res.json().get("isObservation", False)
+                if not is_observation_in_recorded_session:
+                    msg = "You have to specify --observation option to use non-blocking mode"
+                    click.echo(
+                        click.style(
+                            msg,
+                            fg="red"),
+                        err=True,
+                    )
+                    tracking_client.send_error_event(
+                        event_name=Tracking.ErrorEvent.INTERNAL_CLI_ERROR,
+                        stack_trace=msg,
+                    )
+                    sys.exit(1)
+            except Exception as e:
+                tracking_client.send_error_event(
+                    event_name=Tracking.ErrorEvent.INTERNAL_CLI_ERROR,
+                    stack_trace=str(e),
+                )
+                click.echo(ignorable_error(e), err=True)
+
     file_path_normalizer = FilePathNormalizer(base_path, no_base_path_inference=no_base_path_inference)
 
     # TODO: placed here to minimize invasion in this PR to reduce the likelihood of
@@ -200,14 +377,15 @@ def subset(
         # exclusion_output_handler: Callable[[List[TestPathLike],
         # List[TestPathLike], bool], None]]
 
-        def __init__(self, dry_run: bool = False):
+        def __init__(self, app: Application):
             self.rest = rest
-            self.test_paths = []  # type: List
+            self.input_given = False  # set to True when an attempt was made to add to self.test_paths
+            self.test_paths: List[List[Dict[str, str]]] = []
             self.output_handler = self._default_output_handler
             self.exclusion_output_handler = self._default_exclusion_output_handler
             self.is_get_tests_from_previous_sessions = is_get_tests_from_previous_sessions
             self.is_output_exclusion_rules = is_output_exclusion_rules
-            super(Optimize, self).__init__(dry_run=dry_run)
+            super(Optimize, self).__init__(app=app)
 
         def _default_output_handler(self, output: List[TestPath], rests: List[TestPath]):
             if rest:
@@ -220,20 +398,21 @@ def subset(
             self.output_handler(rest, subset)
 
         def test_path(self, path: TestPathLike):
+            """register one test"""
+
             def rel_base_path(path):
                 if isinstance(path, str):
                     return pathlib.Path(file_path_normalizer.relativize(path)).as_posix()
                 else:
                     return path
 
+            self.input_given = True
             if isinstance(path, str) and any(s in path for s in ('*', "?")):
                 for i in glob.iglob(path, recursive=True):
                     if os.path.isfile(i):
                         self.test_paths.append(self.to_test_path(rel_base_path(i)))
-                return
-
-            """register one test"""
-            self.test_paths.append(self.to_test_path(rel_base_path(path)))
+            else:
+                self.test_paths.append(self.to_test_path(rel_base_path(path)))
 
         def stdin(self) -> Union[TextIO, List]:
             # To avoid the cli continue to wait from stdin
@@ -247,12 +426,10 @@ def subset(
             they didn't feed anything from stdin
             """
             if sys.stdin.isatty():
-                click.echo(
-                    click.style(
-                        "Warning: this command reads from stdin but it doesn't appear to be connected to anything. "
-                        "Did you forget to pipe from another command?",
-                        fg='yellow'),
-                    err=True)
+                warn_and_exit_if_fail_fast_mode(
+                    "Warning: this command reads from stdin but it doesn't appear to be connected to anything. "
+                    "Did you forget to pipe from another command?"
+                )
             return sys.stdin
 
         @staticmethod
@@ -280,6 +457,8 @@ def subset(
                 - if a TestPath is returned, that's added as is
             """
 
+            self.input_given = True
+
             if path_builder is None:
                 # default implementation of path_builder creates a file name relative to `source` so as not
                 # to be affected by the path
@@ -302,7 +481,7 @@ def subset(
             duration: Optional[DurationType],
             test_runner: str,
         ):
-            payload = {
+            payload: Dict[str, Any] = {
                 "testPaths": self.test_paths,
                 "testRunner": test_runner,
                 "session": {
@@ -328,22 +507,33 @@ def subset(
                     "type": "subset-by-confidence",
                     "percentage": confidence
                 }
+            elif goal_spec is not None:
+                payload["goal"] = {
+                    "type": "subset-by-goal-spec",
+                    "goal": goal_spec
+                }
             else:
                 payload['useServerSideOptimizationTarget'] = True
 
             if ignore_flaky_tests_above:
                 payload["dropFlakinessThreshold"] = ignore_flaky_tests_above
 
+            if prioritize_tests_failed_within_hours:
+                payload["hoursToPrioritizeFailedTest"] = prioritize_tests_failed_within_hours
+
+            if prioritized_tests_mapping_file:
+                payload['prioritizedTestsMapping'] = json.load(prioritized_tests_mapping_file)
+
             return payload
 
         def run(self):
             """called after tests are scanned to compute the optimized order"""
             if not is_get_tests_from_previous_sessions and len(self.test_paths) == 0:
-                click.echo(
-                    click.style(
-                        "ERROR: subset candidates are empty. Please set subset candidates or use `--get-tests-from-previous-sessions` option",  # noqa E501
-                        fg="red"),
-                    err=True)
+                if self.input_given:
+                    msg = "ERROR: Given arguments did not match any tests. They appear to be incorrect/non-existent."  # noqa E501
+                else:
+                    msg = "ERROR: Expecting tests to be given, but none provided. See https://www.launchableinc.com/docs/features/predictive-test-selection/requesting-and-running-a-subset-of-tests/subsetting-with-the-launchable-cli/ and provide ones, or use the `--get-tests-from-previous-sessions` option"  # noqa E501
+                click.echo(click.style(msg, fg="red"), err=True)
                 exit(1)
 
             # When Error occurs, return the test name as it is passed.
@@ -361,15 +551,34 @@ def subset(
             else:
                 try:
                     test_runner = context.invoked_subcommand
-                    client = LaunchableClient(test_runner=test_runner, dry_run=context.obj.dry_run)
 
                     # temporarily extend the timeout because subset API response has become slow
                     # TODO: remove this line when API response return respose
-                    # within 60 sec
-                    timeout = (5, 180)
+                    # within 300 sec
+                    timeout = (5, 300)
                     payload = self.get_payload(session_id, target, duration, test_runner)
 
-                    res = client.request("post", "subset", timeout=timeout, payload=payload, compress=True)
+                    if is_non_blocking:
+                        # Create a new process for requesting a subset.
+                        process = Process(target=subset_request, args=(client, timeout, payload))
+                        process.start()
+                        click.echo("The subset was requested in non-blocking mode.", err=True)
+                        self.output_handler(self.test_paths, [])
+                        return
+
+                    res = subset_request(client=client, timeout=timeout, payload=payload)
+
+                    # The status code 422 is returned when validation error of the test mapping file occurs.
+                    if res.status_code == 422:
+                        msg = "Error: {}".format(res.reason)
+                        tracking_client.send_error_event(
+                            event_name=Tracking.ErrorEvent.USER_ERROR,
+                            stack_trace=msg,
+                        )
+                        click.echo(
+                            click.style(msg, fg="red"),
+                            err=True)
+                        sys.exit(1)
 
                     res.raise_for_status()
 
@@ -381,17 +590,16 @@ def subset(
                     is_observation = res.json().get("isObservation", False)
 
                 except Exception as e:
-                    if os.getenv(REPORT_ERROR_KEY):
-                        raise e
-                    else:
-                        click.echo(e, err=True)
+                    tracking_client.send_error_event(
+                        event_name=Tracking.ErrorEvent.INTERNAL_CLI_ERROR,
+                        stack_trace=str(e),
+                    )
 
-                    click.echo(click.style(
-                        "Warning: the service failed to subset. Falling back to running all tests", fg='yellow'),
-                        err=True)
+                    client.print_exception_and_recover(
+                        e, "Warning: the service failed to subset. Falling back to running all tests")
 
             if len(original_subset) == 0:
-                click.echo(click.style("Error: no tests found matching the path.", 'yellow'), err=True)
+                warn_and_exit_if_fail_fast_mode("Error: no tests found matching the path.")
                 return
 
             if split:
@@ -464,4 +672,8 @@ def subset(
                 "\nRun `launchable inspect subset --subset-id {}` to view full subset details".format(subset_id),
                 err=True)
 
-    context.obj = Optimize(dry_run=context.obj.dry_run)
+    context.obj = Optimize(app=context.obj)
+
+
+def subset_request(client: LaunchableClient, timeout: Tuple[int, int], payload: Dict[str, Any]):
+    return client.request("post", "subset", timeout=timeout, payload=payload, compress=True)
