@@ -3,6 +3,8 @@ from typing import Dict, List
 
 import click
 
+from launchable.utils.java import junit5_nested_class_path_builder
+
 from ..utils.file_name_pattern import jvm_test_pattern
 from . import launchable
 
@@ -104,4 +106,8 @@ def to_class_file(class_name: str):
     return class_name.replace('.', '/') + '.class'
 
 
-record_tests = launchable.CommonRecordTestImpls(__name__).report_files()
+@click.argument('reports', required=True, nargs=-1)
+@launchable.record.tests
+def record_tests(client, reports):
+    client.path_builder = junit5_nested_class_path_builder(client.path_builder)
+    launchable.CommonRecordTestImpls.load_report_files(client=client, source_roots=reports)
