@@ -9,11 +9,10 @@ from . import launchable
 
 
 def parse_func(p: str) -> ET.ElementTree:
-    datetime_format = '%Y%m%d %H:%M:%S.%f'
 
-    original_tree = ET.parse(p)
-    testsuite = ET.Element("testsuite", {"name": "robot"})
-    for suite in original_tree.findall("suite/suite"):
+    def parse_suite(suite: ET.Element):
+        DATETIME_FORMAT = '%Y%m%d %H:%M:%S.%f'
+
         suite_name = suite.get('name')
         for test in suite.iter("test"):
             test_name = test.get('name')
@@ -30,15 +29,15 @@ def parse_func(p: str) -> ET.ElementTree:
                 end_time_str = status_node.get('endtime') if status_node is not None else ''
 
                 if start_time_str != '' and end_time_str != '':
-                    start_time = datetime.strptime(str(start_time_str), datetime_format)
-                    end_time = datetime.strptime(str(end_time_str), datetime_format)
+                    start_time = datetime.strptime(str(start_time_str), DATETIME_FORMAT)
+                    end_time = datetime.strptime(str(end_time_str), DATETIME_FORMAT)
 
                     duration = end_time - start_time
 
                 testcase = ET.SubElement(testsuite, "testcase", {
                     "name": str(test_name),
                     "classname": str(suite_name),
-                    "time": str(duration.microseconds / 1000 / 1000) if duration is not None else '0',
+                    "time": str(duration.total_seconds()) if duration is not None else '0',
                 })
 
                 if status == "FAIL":
@@ -49,6 +48,21 @@ def parse_func(p: str) -> ET.ElementTree:
                 if status == "NOT_RUN" or nested_status == 'NOT_RUN':
                     skipped = ET.SubElement(testcase, "skipped")  # noqa: F841
 
+    original_tree = ET.parse(p)
+    testsuite = ET.Element("testsuite", {"name": "robot"})
+
+    SUITE_TAG_NAME = "suite"
+    for suites in original_tree.findall(SUITE_TAG_NAME):
+        nested_suites = suites.findall(SUITE_TAG_NAME)
+
+        if nested_suites:
+            # Run tests in a directory
+            for suite in nested_suites:
+                parse_suite(suite)
+        else:
+            # Run tests in a single file
+            parse_suite(suites)
+
     return ET.ElementTree(testsuite)
 
 
@@ -66,13 +80,14 @@ def record_tests(client, reports):
 @launchable.subset
 def subset(client, reports):
     for r in reports:
-        suite = JUnitXml.fromfile(r, parse_func)
+        xml = JUnitXml.fromfile(r, parse_func)
 
-        for case in suite:
-            cls_name = case._elem.attrib.get("classname")
-            name = case._elem.attrib.get('name')
-            if cls_name != '' and name != '':
-                client.test_path([{'type': 'class', 'name': cls_name}, {'type': 'testcase', 'name': name}])
+        for suite in xml:
+            for case in suite:
+                cls_name = case._elem.attrib.get("classname")
+                name = case._elem.attrib.get('name')
+                if cls_name != '' and name != '':
+                    client.test_path([{'type': 'class', 'name': cls_name}, {'type': 'testcase', 'name': name}])
 
     client.formatter = robot_formatter
     client.separator = " "
