@@ -1,8 +1,12 @@
+import datetime
 import re
 import sys
-from typing import Dict, Optional
+from typing import Dict, Optional, Tuple
 
 import click
+import dateutil.parser
+from click import ParamType
+from dateutil.tz import tzlocal
 
 # click.Group has the notion of hidden commands but it doesn't allow us to easily add
 # the same command under multiple names and hide all but one.
@@ -11,7 +15,7 @@ import click
 class GroupWithAlias(click.Group):
     def __init__(self, name: Optional[str] = None, commands: Optional[Dict[str, click.Command]] = None, **attrs):
         super().__init__(name, commands, **attrs)
-        self.aliases = {}  # type: Dict[str, str]
+        self.aliases: Dict[str, str] = {}
 
     def get_command(self, ctx: click.core.Context, cmd_name: str):
         return super().get_command(ctx, cmd_name) or self.aliases.get(cmd_name)
@@ -20,7 +24,7 @@ class GroupWithAlias(click.Group):
         self.aliases[name] = cmd
 
 
-class PercentageType(click.ParamType):
+class PercentageType(ParamType):
     name = "percentage"
 
     def convert(self, value: str, param: Optional[click.core.Parameter], ctx: Optional[click.core.Context]):
@@ -35,7 +39,7 @@ class PercentageType(click.ParamType):
         self.fail("Expected percentage like 50% but got '{}'".format(value), param, ctx)
 
 
-class DurationType(click.ParamType):
+class DurationType(ParamType):
     name = "duration"
 
     def convert(self, value: str, param: Optional[click.core.Parameter], ctx: Optional[click.core.Context]):
@@ -48,64 +52,34 @@ class DurationType(click.ParamType):
         self.fail("Expected duration like 3600, 30m, 1h15m but got '{}'".format(value), param, ctx)
 
 
-class KeyValueType(click.Option):
+class KeyValueType(ParamType):
+    name = "key=value"
+
     '''
     Handles options that take key/value pairs.
 
     The preferred syntax is "--option key=value" and that's what we should be advertising in docs and help,
-    but for compatibility (?) we accept other forms of "--option key:value" or "--option key value"
+    but for compatibility (?) we accept "--option key:value"
+
+    Typically, this is used with multiple=True to produce `Sequence[Tuple[str, str]]`.
     '''
-    error_message = "Expected a key-value pair formatted as --option key=value, --option key:value, " \
-                    "or --option key value, but got '{}'"
+    error_message = "Expected a key-value pair formatted as --option key=value, but got '{}'"
 
-    def __init__(self, *args, **kwargs):
-        super(KeyValueType, self).__init__(*args, **kwargs)
-        self._previous_parser_process = None
-        self._key_value_parser = None
+    def convert(
+            self, value: str, param: Optional[click.core.Parameter], ctx: Optional[click.core.Context]
+    ) -> Tuple[str, str]:
 
-    def add_to_parser(self, parser, ctx: click.core.Context):
-        def parser_process(value, state):
-            # case: --option key=value
-            if '=' in value:
-                kv = value.split('=')
+        for delimiter in ['=', ':']:
+            if delimiter in value:
+                kv = value.split(delimiter, 1)
                 if len(kv) != 2:
-                    raise ValueError(self.error_message.format(value))
+                    self.fail(self.error_message.format(value))
+                return kv[0].strip(), kv[1].strip()
 
-                value = tuple([kv[0].strip(), kv[1].strip()])
-            # case: --option key:value
-            elif ':' in value:
-                kv = value.split(':')
-                if len(kv) != 2:
-                    raise ValueError(self.error_message.format(value))
-
-                value = tuple([kv[0].strip(), kv[1].strip()])
-            # case: --option key value
-            else:
-                rargs = state.rargs
-                # --option key-only
-                if len(rargs) < 1:
-                    raise ValueError(self.error_message.format(value))
-                # --option key --other-option / -option key - other-argument
-                elif 0 < len(rargs) and any(rargs[0].startswith(p) for p in self._key_value_parser.prefixes):
-                    raise ValueError(self.error_message.format(" ".join([value, rargs[0]])))
-
-                value = [value, state.rargs.pop(0)]
-
-            self._previous_parser_process(tuple([value[0], value[1]]), state)
-
-        retval = super(KeyValueType, self).add_to_parser(parser, ctx)
-        for name in self.opts:
-            our_parser = parser._long_opt.get(name) or parser._short_opt.get(name)
-            if our_parser:
-                self._key_value_parser = our_parser
-                self._previous_parser_process = our_parser.process
-                our_parser.process = parser_process
-                break
-
-        return retval
+        self.fail(self.error_message.format(value))
 
 
-class FractionType (click.ParamType):
+class FractionType(ParamType):
     name = "fraction"
 
     def convert(self, value: str, param: Optional[click.core.Parameter], ctx: Optional[click.core.Context]):
@@ -123,9 +97,24 @@ class FractionType (click.ParamType):
         self.fail("Expected fraction like 1/2 but got '{}'".format(value), param, ctx)
 
 
+class DateTimeWithTimezoneType(ParamType):
+    name = "datetime"
+
+    def convert(self, value: str, param: Optional[click.core.Parameter], ctx: Optional[click.core.Context]):
+        try:
+            dt = dateutil.parser.parse(value)
+            if dt.tzinfo is None:
+                return dt.replace(tzinfo=tzlocal())
+            return dt
+        except ValueError:
+            self.fail("Expected datetime like 2023-10-01T12:00:00 but got '{}'".format(value), param, ctx)
+
+
 PERCENTAGE = PercentageType()
 DURATION = DurationType()
 FRACTION = FractionType()
+KEY_VALUE = KeyValueType()
+DATETIME_WITH_TZ = DateTimeWithTimezoneType()
 
 # Can the output deal with Unicode emojis?
 try:
@@ -168,3 +157,24 @@ def convert_to_seconds(s: str):
         duration += int(val) * u
 
     return float(duration)
+
+
+def ignorable_error(e: Exception):
+    return "An error occurred on Launchable CLI. You can ignore this message since the process will continue. Error: {}".format(e)
+
+
+def validate_past_datetime(ctx, param, value):
+    """
+    Validates that the provided datetime is in the past.
+    """
+    if value is None:
+        return value
+
+    if not isinstance(value, datetime.datetime):
+        raise click.BadParameter("Expected a datetime object.")
+
+    now = datetime.datetime.now(tz=tzlocal())
+    if value >= now:
+        raise click.BadParameter("The provided datetime must be in the past. But the value is {}".format(value))
+
+    return value
