@@ -1,11 +1,12 @@
 package com.launchableinc.ingest.commits;
 
-import ch.qos.logback.classic.Level;
 import com.google.common.annotations.VisibleForTesting;
 import java.io.File;
 import java.io.IOException;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.util.logging.Logger;
+import java.util.logging.Level;
 import org.eclipse.jgit.api.Git;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.lib.RepositoryBuilder;
@@ -14,8 +15,6 @@ import org.kohsuke.args4j.Argument;
 import org.kohsuke.args4j.CmdLineException;
 import org.kohsuke.args4j.CmdLineParser;
 import org.kohsuke.args4j.Option;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
 
 /** Driver for {@link CommitGraphCollector}. */
 public class CommitIngester {
@@ -38,6 +37,9 @@ public class CommitIngester {
   @Option(name = "-dry-run", usage = "Instead of actually sending data, print what it would do.")
   public boolean dryRun;
 
+  @Option(name = "-skip-cert-verification", usage = "Bypass SSL certification verification.")
+  public boolean skipCertVerification;
+
   /**
    * @deprecated this is an old option and this is on always.
    */
@@ -45,13 +47,12 @@ public class CommitIngester {
   @Option(name = "-scrub-pii", usage = "Scrub emails and names", hidden = true)
   public boolean scrubPii;
 
-  /**
-   * @deprecated this is an old option and this is off always.
-   */
-  @Deprecated
   @Option(name = "-commit-message", usage = "Collect commit messages")
   public boolean commitMessage;
 
+  @Option(name = "-files", usage = "Collect files")
+  public boolean collectFiles;
+
   @Option(
       name = "-max-days",
       usage = "The maximum number of days to collect commits retroactively.")
@@ -60,6 +61,9 @@ public class CommitIngester {
   @Option(name = "-audit", usage = "Whether to output the audit log or not")
   public boolean audit;
 
+  @Option(name = "-enable-timeout", usage = "Enable timeout for the HTTP requests")
+  public boolean enableTimeout;
+
   private Authenticator authenticator;
 
   @VisibleForTesting String launchableToken = null;
@@ -129,7 +133,10 @@ public class CommitIngester {
   }
 
   @VisibleForTesting
-  void run() throws CmdLineException, MalformedURLException, IOException {
+  void run() throws CmdLineException, IOException {
+    if (skipCertVerification) {
+      SSLBypass.install();
+    }
     parseConfiguration();
 
     URL endpoint = new URL(url, String.format("organizations/%s/workspaces/%s/commits/", org, ws));
@@ -140,13 +147,15 @@ public class CommitIngester {
       cgc.setMaxDays(maxDays);
       cgc.setAudit(audit);
       cgc.setDryRun(dryRun);
-      cgc.transfer(endpoint, authenticator);
+      cgc.collectCommitMessage(commitMessage);
+      cgc.collectFiles(collectFiles);
+      cgc.transfer(endpoint, authenticator, enableTimeout);
       int numCommits = cgc.getCommitsSent();
       String suffix = "commit";
       if (numCommits != 1) {
         suffix = "commits";
       }
-      System.out.printf("Launchable recorded %d %s from repository %s%n", numCommits, suffix, repo);
+      System.out.printf("Launchable transferred %d more %s from repository %s%n", numCommits, suffix, repo);
     }
   }
 
@@ -184,10 +193,7 @@ public class CommitIngester {
   static {
     // JGit uses high logging level for errors that it recovers, and those messages are confusing
     // to users. So let's shut them off
-    Logger logger = LoggerFactory.getLogger("org.eclipse.jgit");
-    if (logger instanceof ch.qos.logback.classic.Logger) {
-      ch.qos.logback.classic.Logger logbackLogger = (ch.qos.logback.classic.Logger) logger;
-      logbackLogger.setLevel(Level.OFF);
-    }
+    Logger logger = Logger.getLogger("org.eclipse.jgit");
+    logger.setLevel(Level.OFF);
   }
 }
