@@ -42,7 +42,7 @@ class SplitSubsetTest(CliTestCase):
         rest = tempfile.NamedTemporaryFile(delete=False)
         result = self.cli("split-subset", "--subset-id", "subset/456", "--bin", "1/2", "--rest",
                           rest.name, "file", mix_stderr=False, input=pipe)
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
         self.assertEqual(result.stdout, "test_1.py\ntest_3.py\n")
         self.assertEqual(rest.read().decode(), os.linesep.join(["test_5.py"]))
         rest.close()
@@ -63,9 +63,9 @@ class SplitSubsetTest(CliTestCase):
         result = self.cli("split-subset", "--subset-id", "subset/456", "--bin", "1/2", "--rest",
                           observation_mode_rest.name, "file", mix_stderr=False, input=pipe)
 
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
         self.assertEqual(result.stdout, "test_1.py\ntest_3.py\ntest_5.py\n")
-        self.assertEqual(observation_mode_rest.read().decode(), os.linesep.join(["test_5.py"]))
+        self.assertEqual(observation_mode_rest.read().decode(), "")
         observation_mode_rest.close()
         os.unlink(observation_mode_rest.name)
 
@@ -103,7 +103,7 @@ class SplitSubsetTest(CliTestCase):
             mix_stderr=False,
             input=pipe,
         )
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
         self.assertEqual(result.stdout, "test_1.py\ntest_6.py\n")
 
     @responses.activate
@@ -146,70 +146,65 @@ class SplitSubsetTest(CliTestCase):
             ]
         }
 
-        responses.replace(
-            responses.POST,
-            "{base_url}/intake/organizations/{organization}/workspaces/{workspace}/subset/{subset_id}/split-by-groups".format(
-                base_url=get_base_url(),
-                organization=self.organization,
-                workspace=self.workspace,
-                subset_id=self.subsetting_id,
-            ),
-            json=mock_json_response,
-            status=200
-        )
+        """
+         Note(Konboi):
+            Don't know the cause, but in the Python 3.10 environment,
+            the settings configured with responses.replace disappear on the second call.
+            see: https://github.com/launchableinc/cli/actions/runs/11697720998/job/32576899978#step:10:88
+            So, to call it each time, `replace_response` was defined.
+        """
+        def replace_response():
+            responses.replace(
+                responses.POST,
+                "{base_url}/intake/organizations/{organization}/workspaces/{workspace}/subset/{subset_id}/split-by-groups".format(
+                    base_url=get_base_url(),
+                    organization=self.organization,
+                    workspace=self.workspace,
+                    subset_id=self.subsetting_id,
+                ),
+                json=mock_json_response,
+                status=200
+            )
 
         with tempfile.TemporaryDirectory() as tmpdir:
+            replace_response()
             result = self.cli("split-subset", "--subset-id", "subset/{}".format(self.subsetting_id),
                               "--split-by-groups", "--split-by-groups-output-dir", tmpdir, "file")
 
-            self.assertEqual(result.exit_code, 0)
-            with open("{}/subset-e2e.txt".format(tmpdir)) as f:
-                self.assertEqual(f.read(), "e2e-aaa.py\ne2e-bbb.py")
+            self.assert_contents("{}/subset-e2e.txt".format(tmpdir), "e2e-aaa.py\ne2e-bbb.py")
+            self.assert_contents("{}/subset-{}.txt".format(tmpdir, SPLIT_BY_GROUPS_NO_GROUP_NAME), "aaa.py\nbbb.py")
+            # check the group file
+            self.assert_contents("{}/{}".format(tmpdir, SPLIT_BY_GROUP_SUBSET_GROUPS_FILE_NAME), "e2e")
 
-            self.assertFalse(os.path.exists("{}/rest-e2e.txt".format(tmpdir)))
+            # server doesn't return subset of unit-test
+            self.assert_file_exists("{}/subset-unit-test.txt".format(tmpdir), False)
 
-            self.assertFalse(os.path.exists("{}/subset-unit-test.txt".format(tmpdir)))
-
-            self.assertFalse(os.path.exists("{}/rest-unit-test.txt".format(tmpdir)))
-
-            with open("{}/subset-{}.txt".format(tmpdir, SPLIT_BY_GROUPS_NO_GROUP_NAME)) as f:
-                self.assertEqual(f.read(), "aaa.py\nbbb.py")
-
-            self.assertFalse(os.path.exists("{}/rest-{}.txt".format(tmpdir, SPLIT_BY_GROUPS_NO_GROUP_NAME)))
-
-            with open("{}/{}".format(tmpdir, SPLIT_BY_GROUP_SUBSET_GROUPS_FILE_NAME)) as f:
-                self.assertEqual(f.read(), "e2e")
-
-            self.assertFalse(os.path.exists("{}/{}".format(tmpdir, SPLIT_BY_GROUP_REST_GROUPS_FILE_NAME)))
+            # doesn't set the --rest option
+            self.assert_file_exists("{}/rest-e2e.txt".format(tmpdir), False)
+            self.assert_file_exists("{}/rest-unit-test.txt".format(tmpdir), False)
+            self.assert_file_exists("{}/rest-{}.txt".format(tmpdir, SPLIT_BY_GROUPS_NO_GROUP_NAME), False)
+            self.assert_file_exists("{}/{}".format(tmpdir, SPLIT_BY_GROUP_REST_GROUPS_FILE_NAME), False)
 
         # with rest option
         with tempfile.TemporaryDirectory() as tmpdir:
+            replace_response()
             result = self.cli("split-subset", "--subset-id", "subset/{}".format(self.subsetting_id),
-                              "--split-by-groups-with-rest", "--split-by-groups-output-dir", tmpdir, "file")
+                              "--split-by-groups-with-rest", "--split-by-groups-output-dir", tmpdir, "file", mix_stderr=False)
 
-            self.assertEqual(result.exit_code, 0)
-            with open("{}/subset-e2e.txt".format(tmpdir)) as f:
-                self.assertEqual(f.read(), "e2e-aaa.py\ne2e-bbb.py")
+            self.assert_success(result)
+            self.assert_contents("{}/subset-e2e.txt".format(tmpdir), "e2e-aaa.py\ne2e-bbb.py")
+            self.assert_contents("{}/rest-e2e.txt".format(tmpdir), "e2e-ccc.py\ne2e-ddd.py")
 
-            with open("{}/rest-e2e.txt".format(tmpdir)) as f:
-                self.assertEqual(f.read(), "e2e-ccc.py\ne2e-ddd.py")
+            # server doesn't return subset of unit-test
+            self.assert_file_exists("{}/subset-unit-test.txt".format(tmpdir), False)
+            self.assert_contents("{}/rest-unit-test.txt".format(tmpdir), "unit-test-111.py\nunit-test-222.py")
 
-            self.assertFalse(os.path.exists("{}/subset-unit-test.txt".format(tmpdir)))
+            self.assert_contents("{}/subset-{}.txt".format(tmpdir, SPLIT_BY_GROUPS_NO_GROUP_NAME), "aaa.py\nbbb.py")
+            self.assert_contents("{}/rest-{}.txt".format(tmpdir, SPLIT_BY_GROUPS_NO_GROUP_NAME), "111.py\n222.py")
 
-            with open("{}/rest-unit-test.txt".format(tmpdir)) as f:
-                self.assertEqual(f.read(), "unit-test-111.py\nunit-test-222.py")
-
-            with open("{}/subset-{}.txt".format(tmpdir, SPLIT_BY_GROUPS_NO_GROUP_NAME)) as f:
-                self.assertEqual(f.read(), "aaa.py\nbbb.py")
-
-            with open("{}/rest-{}.txt".format(tmpdir, SPLIT_BY_GROUPS_NO_GROUP_NAME)) as f:
-                self.assertEqual(f.read(), "111.py\n222.py")
-
-            with open("{}/{}".format(tmpdir, SPLIT_BY_GROUP_SUBSET_GROUPS_FILE_NAME)) as f:
-                self.assertEqual(f.read(), "e2e")
-
-            with open("{}/{}".format(tmpdir, SPLIT_BY_GROUP_REST_GROUPS_FILE_NAME)) as f:
-                self.assertEqual(f.read(), "unit-test")
+            # check the group file
+            self.assert_contents("{}/{}".format(tmpdir, SPLIT_BY_GROUP_SUBSET_GROUPS_FILE_NAME), "e2e")
+            self.assert_contents("{}/{}".format(tmpdir, SPLIT_BY_GROUP_REST_GROUPS_FILE_NAME), "unit-test")
 
     @responses.activate
     @mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
@@ -267,20 +262,16 @@ class SplitSubsetTest(CliTestCase):
             result = self.cli("split-subset", "--subset-id", "subset/{}".format(self.subsetting_id),
                               "--split-by-groups", "--split-by-groups-output-dir", tmpdir, '--output-exclusion-rules', "file")
 
-            self.assertEqual(result.exit_code, 0)
-            with open("{}/subset-e2e.txt".format(tmpdir)) as f:
-                self.assertCountEqual(f.read().splitlines(), ["e2e-ccc.py", "e2e-ddd.py", ])
+            self.assert_success(result)
 
-            self.assertFalse(os.path.exists("{}/rest-e2e.txt".format(tmpdir)))
+            # --output-exclusion-rules is enabled, thus switched subset and rest
+            self.assert_contents("{}/subset-e2e.txt".format(tmpdir), "e2e-ccc.py\ne2e-ddd.py")
+            self.assert_contents("{}/subset-unit-test.txt".format(tmpdir), "unit-test-111.py\nunit-test-222.py")
+            self.assert_contents("{}/subset-{}.txt".format(tmpdir, SPLIT_BY_GROUPS_NO_GROUP_NAME), "111.py\n222.py")
+            self.assert_contents("{}/{}".format(tmpdir, SPLIT_BY_GROUP_SUBSET_GROUPS_FILE_NAME), "unit-test")
 
-            with open("{}/subset-unit-test.txt".format(tmpdir)) as f:
-                self.assertCountEqual(f.read().splitlines(), ["unit-test-111.py", "unit-test-222.py"])
-            self.assertFalse(os.path.exists("{}/rest-unit-test.txt".format(tmpdir)))
-
-            with open("{}/subset-{}.txt".format(tmpdir, SPLIT_BY_GROUPS_NO_GROUP_NAME)) as f:
-                self.assertCountEqual(f.read().splitlines(), ["111.py", "222.py"])
-            self.assertFalse(os.path.exists("{}/rest-{}.txt".format(tmpdir, SPLIT_BY_GROUPS_NO_GROUP_NAME)))
-
-            with open("{}/{}".format(tmpdir, SPLIT_BY_GROUP_SUBSET_GROUPS_FILE_NAME)) as f:
-                self.assertEqual(f.read(), "unit-test")
-            self.assertFalse(os.path.exists("{}/{}".format(tmpdir, SPLIT_BY_GROUP_REST_GROUPS_FILE_NAME)))
+            # doesn't set the --rest option
+            self.assert_file_exists("{}/rest-e2e.txt".format(tmpdir), False)
+            self.assert_file_exists("{}/rest-unit-test.txt".format(tmpdir), False)
+            self.assert_file_exists("{}/rest-{}.txt".format(tmpdir, SPLIT_BY_GROUPS_NO_GROUP_NAME), False)
+            self.assert_file_exists("{}/{}".format(tmpdir, SPLIT_BY_GROUP_REST_GROUPS_FILE_NAME), False)
