@@ -44,7 +44,7 @@ class SubsetTest(CliTestCase):
         rest = tempfile.NamedTemporaryFile(delete=False)
         result = self.cli("subset", "--target", "30%", "--session",
                           self.session, "--rest", rest.name, "file", mix_stderr=False, input=pipe)
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
         self.assertEqual(result.stdout, "test_1.py\ntest_2.py\n")
         self.assertEqual(rest.read().decode(), os.linesep.join(["test_3.py", "test_4.py"]))
         rest.close()
@@ -77,7 +77,7 @@ class SubsetTest(CliTestCase):
         rest = tempfile.NamedTemporaryFile(delete=False)
         result = self.cli("subset", "--target", "30%", "--session",
                           self.session, "--rest", rest.name, "file", mix_stderr=False, input=pipe)
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
         self.assertEqual(result.stdout, "test_1.py\ntest_2.py\ntest_3.py\ntest_4.py\n")
         self.assertEqual(rest.read().decode(), "")
         rest.close()
@@ -126,7 +126,7 @@ class SubsetTest(CliTestCase):
             "file",
             input=pipe,
             mix_stderr=False)
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
 
         self.assertEqual(result.stdout, "test_1.py\ntest_2.py\ntest_3.py\ntest_4.py\n")
         self.assertEqual(observation_mode_rest.read().decode(), "")
@@ -160,7 +160,7 @@ class SubsetTest(CliTestCase):
         rest = tempfile.NamedTemporaryFile(delete=False)
         result = self.cli("subset", "--target", "30%", "--session",
                           self.session, "--rest", rest.name, "--observation", "file", mix_stderr=False, input=pipe)
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
         self.assertEqual(result.stdout, "test_1.py\ntest_2.py\ntest_3.py\ntest_4.py\n")
         self.assertEqual(rest.read().decode(), "")
         rest.close()
@@ -203,11 +203,44 @@ class SubsetTest(CliTestCase):
             "file",
             input=pipe,
             mix_stderr=False)
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
 
-        payload = json.loads(gzip.decompress(responses.calls[0].request.body).decode())
+        payload = json.loads(gzip.decompress(responses.calls[1].request.body).decode())
         self.assertTrue(payload.get('useServerSideOptimizationTarget'))
 
+    @responses.activate
+    @mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
+    def test_subset_goalspec(self):
+        # make sure --goal-spec gets translated properly to a JSON request payload
+        responses.replace(
+            responses.POST,
+            "{}/intake/organizations/{}/workspaces/{}/subset".format(
+                get_base_url(),
+                self.organization,
+                self.workspace),
+            json={
+                "testPaths": [
+                    [{"type": "file", "name": "test_aaa.py"}],
+                ],
+                "testRunner": "file",
+                "rest": [],
+                "subsettingId": 123,
+            },
+            status=200)
+
+        result = self.cli(
+            "subset",
+            "--session",
+            self.session,
+            "--goal-spec",
+            "foo(),bar(zot=3%)",
+            "file",
+            input="test_aaa.py")
+        self.assert_success(result)
+
+        payload = json.loads(gzip.decompress(responses.calls[1].request.body).decode())
+        self.assertEqual(payload.get('goal').get('goal'), "foo(),bar(zot=3%)")
+
     @responses.activate
     @mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
     def test_subset_ignore_flaky_tests_above(self):
@@ -245,18 +278,18 @@ class SubsetTest(CliTestCase):
             "file",
             input=pipe,
             mix_stderr=False)
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
 
-        payload = json.loads(gzip.decompress(responses.calls[0].request.body).decode())
+        payload = json.loads(gzip.decompress(responses.calls[1].request.body).decode())
         self.assertEqual(payload.get('dropFlakinessThreshold'), 0.05)
 
-    @ responses.activate
-    @ mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
+    @responses.activate
+    @mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
     def test_subset_with_get_tests_from_previous_full_runs(self):
         # check error when input candidates are empty without --get-tests-from-previous-sessions option
         result = self.cli("subset", "--target", "30%", "--session", self.session, "file")
-        self.assertEqual(result.exit_code, 1)
-        self.assertTrue("Please set subset candidates or use `--get-tests-from-previous-sessions` option" in result.stdout)
+        self.assert_exit_code(result, 1)
+        self.assertIn("use the `--get-tests-from-previous-sessions` option", result.stdout)
 
         responses.replace(
             responses.POST,
@@ -297,14 +330,14 @@ class SubsetTest(CliTestCase):
             "file",
             mix_stderr=False)
 
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
         self.assertEqual(result.stdout, "test_aaa.py\ntest_bbb.py\ntest_ccc.py\n")
         self.assertEqual(rest.read().decode(), os.linesep.join(["test_eee.py", "test_fff.py", "test_ggg.py"]))
         rest.close()
         os.unlink(rest.name)
 
-    @ responses.activate
-    @ mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
+    @responses.activate
+    @mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
     def test_subset_with_output_exclusion_rules(self):
         pipe = "test_aaa.py\ntest_111.py\ntest_bbb.py\ntest_222.py\ntest_ccc.py\ntest_333.py\n"
         responses.replace(
@@ -345,7 +378,7 @@ class SubsetTest(CliTestCase):
             input=pipe,
             mix_stderr=False)
 
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
         self.assertEqual(result.stdout, "test_aaa.py\ntest_bbb.py\ntest_ccc.py\n")
         self.assertEqual(rest.read().decode(), os.linesep.join(["test_111.py", "test_222.py", "test_333.py"]))
         rest.close()
@@ -365,7 +398,7 @@ class SubsetTest(CliTestCase):
             input=pipe,
             mix_stderr=False)
 
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
         self.assertEqual(result.stdout, "test_111.py\ntest_222.py\ntest_333.py\n")
 
         self.assertEqual(rest.read().decode(), os.linesep.join(["test_aaa.py", "test_bbb.py", "test_ccc.py"]))
@@ -411,7 +444,7 @@ class SubsetTest(CliTestCase):
             input=pipe,
             mix_stderr=False)
 
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
         self.assertEqual(result.stdout, "")
 
         self.assertEqual(rest.read().decode(), os.linesep.join(
@@ -419,8 +452,8 @@ class SubsetTest(CliTestCase):
         rest.close()
         os.unlink(rest.name)
 
-    @ responses.activate
-    @ mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
+    @responses.activate
+    @mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
     def test_subset_with_observation_and_output_exclusion_rules(self):
         pipe = "test_aaa.py\ntest_111.py\ntest_bbb.py\ntest_222.py\ntest_ccc.py\ntest_333.py\n"
         responses.replace(
@@ -464,10 +497,53 @@ class SubsetTest(CliTestCase):
             input=pipe,
             mix_stderr=False)
 
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
         self.assertEqual(result.stdout, "")
+        self.assertIn("WARNING: --observation and --output-exclusion-rules are set.", result.stderr)
 
         self.assertEqual(rest.read().decode(), os.linesep.join(
             ["test_aaa.py", "test_bbb.py", "test_ccc.py", "test_111.py", "test_222.py", "test_333.py"]))
         rest.close()
         os.unlink(rest.name)
+
+    @responses.activate
+    @mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
+    def test_subset_prioritize_tests_failed_within_hours(self):
+        pipe = "test_aaa.py\ntest_bbb.py\ntest_ccc.py\ntest_flaky.py"
+        responses.replace(
+            responses.POST,
+            "{}/intake/organizations/{}/workspaces/{}/subset".format(
+                get_base_url(),
+                self.organization,
+                self.workspace),
+            json={
+                "testPaths": [
+                    [{"type": "file", "name": "test_aaa.py"}],
+                    [{"type": "file", "name": "test_bbb.py"}],
+
+                ],
+                "testRunner": "file",
+                "rest": [
+                    [{"type": "file", "name": "test_ccc.py"}],
+                ],
+                "subsettingId": 123,
+                "summary": {
+                    "subset": {"duration": 20, "candidates": 2, "rate": 67},
+                    "rest": {"duration": 10, "candidates": 1, "rate": 33}
+                },
+            },
+            status=200)
+
+        result = self.cli(
+            "subset",
+            "--session",
+            self.session,
+            "--prioritize-tests-failed-within-hours",
+            24,
+            "file",
+            input=pipe,
+            mix_stderr=False)
+        self.assert_success(result)
+
+        payload = json.loads(gzip.decompress(responses.calls[1].request.body).decode())
+        self.assertEqual(payload.get('hoursToPrioritizeFailedTest'), 24)
