@@ -1,9 +1,6 @@
-import gzip
-import json
 import os
 import sys
 import tempfile
-from pathlib import Path
 from unittest import mock
 
 import responses  # type: ignore
@@ -14,8 +11,6 @@ from tests.cli_test_case import CliTestCase
 
 
 class CTestTest(CliTestCase):
-    test_files_dir = Path(__file__).parent.joinpath('../data/ctest/').resolve()
-
     @responses.activate
     @mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
     def test_subset_multiple_files(self):
@@ -52,7 +47,7 @@ class CTestTest(CliTestCase):
                               '--output-regex-files-dir=' + output_dir,
                               '--output-regex-files-size=32',
                               str(self.test_files_dir.joinpath("ctest_list.json")))
-            self.assertEqual(result.exit_code, 0)
+            self.assert_success(result)
 
             subset_files = []
             rest_files = []
@@ -85,11 +80,8 @@ class CTestTest(CliTestCase):
         write_build(self.build_name)
 
         result = self.cli('subset', '--target', '10%', 'ctest', str(self.test_files_dir.joinpath("ctest_list.json")))
-        self.assertEqual(result.exit_code, 0)
-
-        payload = json.loads(gzip.decompress(responses.calls[1].request.body).decode())
-        expected = self.load_json_from_file(self.test_files_dir.joinpath('subset_result.json'))
-        self.assert_json_orderless_equal(payload, expected)
+        self.assert_success(result)
+        self.assert_subset_payload('subset_result.json')
 
     @responses.activate
     @mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
@@ -98,12 +90,7 @@ class CTestTest(CliTestCase):
         write_build(self.build_name)
 
         result = self.cli('record', 'tests', 'ctest', str(self.test_files_dir) + "/Testing/**/Test.xml")
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
+
         self.assertEqual(read_session(self.build_name), self.session)
-
-        payload = json.loads(gzip.decompress(responses.calls[2].request.body).decode())
-        expected = self.load_json_from_file(self.test_files_dir.joinpath('record_test_result.json'))
-
-        for c in payload['events']:
-            del c['created_at']
-        self.assert_json_orderless_equal(payload, expected)
+        self.assert_record_tests_payload('record_test_result.json')
