@@ -1,7 +1,4 @@
-import gzip
-import json
 import os
-from pathlib import Path
 from unittest import mock
 
 import responses  # type: ignore
@@ -10,8 +7,6 @@ from tests.cli_test_case import CliTestCase
 
 
 class CypressTest(CliTestCase):
-    test_files_dir = Path(__file__).parent.joinpath('../data/cypress/').resolve()
-
     @responses.activate
     @mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
     def test_record_test_cypress(self):
@@ -19,12 +14,8 @@ class CypressTest(CliTestCase):
         # cypress run --reporter junit report.xml
         result = self.cli('record', 'tests', '--session', self.session,
                           'cypress', str(self.test_files_dir) + "/test-result.xml")
-        self.assertEqual(result.exit_code, 0)
-
-        payload = json.loads(gzip.decompress(responses.calls[1].request.body).decode())
-        expected = self.load_json_from_file(self.test_files_dir.joinpath('record_test_result.json'))
-
-        self.assert_json_orderless_equal(expected, payload)
+        self.assert_success(result)
+        self.assert_record_tests_payload('record_test_result.json')
 
     @responses.activate
     @mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
@@ -33,11 +24,8 @@ class CypressTest(CliTestCase):
         # cypress/integration/examples/window.spec.js, so set it
         pipe = "cypress/integration/examples/window.spec.js"
         result = self.cli('subset', '--target', '10%', '--session', self.session, 'cypress', input=pipe)
-        self.assertEqual(result.exit_code, 0)
-
-        payload = json.loads(gzip.decompress(responses.calls[0].request.body).decode())
-        expected = self.load_json_from_file(self.test_files_dir.joinpath('subset_result.json'))
-        self.assert_json_orderless_equal(expected, payload)
+        self.assert_success(result)
+        self.assert_subset_payload('subset_result.json')
 
     @responses.activate
     @mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
@@ -45,5 +33,6 @@ class CypressTest(CliTestCase):
         # parse empty test report XML
         result = self.cli('record', 'tests', '--session', self.session,
                           'cypress', str(self.test_files_dir) + "/empty.xml")
-        self.assertEqual(result.exit_code, 0)
-        self.assertIn("close", responses.calls[2].request.url, "No record request")
+        self.assert_success(result)
+        for call in responses.calls:
+            self.assertFalse(call.request.url.endswith('/events'), 'there should be no calls to the /events endpoint')
