@@ -13,31 +13,24 @@ from tests.helper import ignore_warnings
 
 
 class MinitestTest(CliTestCase):
-    test_files_dir = Path(__file__).parent.joinpath('../data/minitest/').resolve()
-    result_file_path = test_files_dir.joinpath('record_test_result.json')
-
     @responses.activate
     @mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
     def test_record_test_minitest(self):
         result = self.cli('record', 'tests', '--session', self.session, 'minitest', str(self.test_files_dir) + "/")
-        self.assertEqual(result.exit_code, 0)
-
-        payload = json.loads(gzip.decompress(responses.calls[1].request.body).decode())
-
-        expected = self.load_json_from_file(self.result_file_path)
-        self.assert_json_orderless_equal(expected, payload)
+        self.assert_success(result)
+        self.assert_record_tests_payload('record_test_result.json')
 
     @responses.activate
     @mock.patch.dict(os.environ, {"LAUNCHABLE_TOKEN": CliTestCase.launchable_token})
     def test_record_test_minitest_chunked(self):
         result = self.cli('record', 'tests', '--session', self.session,
                           '--post-chunk', 5, 'minitest', str(self.test_files_dir) + "/")
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
 
-        payload1 = json.loads(gzip.decompress(responses.calls[1].request.body).decode())
+        payload1 = json.loads(gzip.decompress(self.find_request('/events').request.body).decode())
         expected1 = self.load_json_from_file(self.test_files_dir.joinpath('record_test_result_chunk1.json'))
 
-        payload2 = json.loads(gzip.decompress(responses.calls[2].request.body).decode())
+        payload2 = json.loads(gzip.decompress(self.find_request('/events', 1).request.body).decode())
         expected2 = self.load_json_from_file(self.test_files_dir.joinpath('record_test_result_chunk2.json'))
 
         # concat 1st and 2nd request for unstable order
@@ -72,7 +65,8 @@ class MinitestTest(CliTestCase):
         result = self.cli('subset', '--target', '20%', '--session', self.session, '--base', str(self.test_files_dir),
                           'minitest', str(self.test_files_dir) + "/test/**/*.rb")
 
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
+
         output = Path(self.test_files_dir, "test", "example_test.rb")
         self.assertIn(str(output), result.output)
 
@@ -82,7 +76,8 @@ class MinitestTest(CliTestCase):
         result = self.cli('subset', '--target', '20%', '--session', self.session, '--base', str(self.test_files_dir),
                           'minitest', str(self.test_files_dir) + "/dummy")
 
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
+
         self.assertTrue("Error: no tests found matching the path." in result.output)
 
     @responses.activate
@@ -110,7 +105,8 @@ class MinitestTest(CliTestCase):
         result = self.cli('subset', '--target', '20%', '--session', self.session, '--base',
                           str(self.test_files_dir), '--split', 'minitest', str(self.test_files_dir) + "/test/**/*.rb")
 
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
+
         self.assertIn('subset/123', result.output)
 
     @responses.activate
@@ -131,9 +127,10 @@ class MinitestTest(CliTestCase):
         result = self.cli('split-subset', '--subset-id', 'subset/456', '--base', str(self.test_files_dir),
                           '--bin', '2/2', '--rest', rest.name, 'minitest')
 
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
+
         output = Path(self.test_files_dir, "test", "example_test.rb")
         self.assertEqual(str(output), result.output.rstrip("\n"))
-        self.assertEqual(rest.read().decode().rstrip("\n"), str(output))
+        self.assertEqual(rest.read().decode().rstrip("\n"), "")
         rest.close()
         os.unlink(rest.name)
