@@ -4,7 +4,9 @@ import os
 import tempfile
 from unittest import mock
 
+import dateutil.parser
 import responses  # type: ignore
+from dateutil.tz import tzlocal
 
 from launchable.utils.http_client import get_base_url
 from launchable.utils.session import write_build
@@ -51,10 +53,10 @@ class RawTest(CliTestCase):
 
             result = self.cli('subset', '--target', '10%',
                               'raw', test_path_file, mix_stderr=False)
-            self.assertEqual(result.exit_code, 0)
+            self.assert_success(result)
 
             # Check request body
-            payload = json.loads(gzip.decompress(responses.calls[1].request.body).decode())
+            payload = json.loads(gzip.decompress(self.find_request('/subset').request.body).decode())
             self.assert_json_orderless_equal(payload, {
                 'testPaths': [
                     [{'type': 'testcase', 'name': 'FooTest.Bar'}],
@@ -111,10 +113,10 @@ class RawTest(CliTestCase):
             "--rest", rest.name,
             'raw',
             mix_stderr=False)
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
 
         # Check request body
-        payload = json.loads(gzip.decompress(responses.calls[1].request.body).decode())
+        payload = json.loads(gzip.decompress(self.find_request('/subset').request.body).decode())
         self.assert_json_orderless_equal(payload, {
             'testPaths': [],
             'testRunner': 'raw',
@@ -137,17 +139,34 @@ class RawTest(CliTestCase):
         with tempfile.TemporaryDirectory() as tempdir:
             test_path_file = os.path.join(tempdir, 'tests.json')
             test_path_file2 = os.path.join(tempdir, 'tests_2.json')
+            test_path_file3 = os.path.join(tempdir, 'tests_3.json')
+            # These timestamp variables are defined for testing timezone.
+            test_ruby_timestamp = dateutil.parser.parse("2021-10-05T12:34:00")
+            test_python_timestamp = dateutil.parser.parse("2021-10-05T12:34:56")
+            test_go_timestamp = dateutil.parser.parse("2021-10-05T12:35:30")
+            test_java_timestamp = dateutil.parser.parse("2021-10-05T12:36:10")
             with open(test_path_file, 'w') as f:
                 f.write('\n'.join([
                     '{',
                     '  "testCases": [',
                     '     {',
-                    '       "testPath": "file=a.py#class=classA",',
+                    '       "testPath": "file=test_ruby.py#class=classA",',
                     '       "duration": 42,',
                     '       "status": "TEST_PASSED",',
                     '       "stdout": "This is stdout",',
                     '       "stderr": "This is stderr",',
-                    '       "createdAt": "2021-10-05T12:34:00"',
+                    f'       "createdAt": "{test_ruby_timestamp}"',
+                    '     }',
+                    '     ,{',
+                    '       "testPath": "file=test_python.py#class=classAA",',
+                    '       "duration": 12,',
+                    '       "status": "TEST_PASSED",',
+                    '       "stdout": "This is stdout",',
+                    '       "stderr": "This is stderr",',
+                    f'       "createdAt": "{test_python_timestamp}",',
+                    '       "data": {',
+                    '         "lineNumber": 5',
+                    '       }',
                     '     }',
                     '  ]',
                     '}',
@@ -158,12 +177,41 @@ class RawTest(CliTestCase):
                     '{',
                     '  "testCases": [',
                     '     {',
-                    '       "testPath": "file=b.py#class=classB",',
+                    '       "testPath": "file=test_go.py#class=classB",',
                     '       "duration": 32,',
                     '       "status": "TEST_PASSED",',
                     '       "stdout": "This is stdout",',
                     '       "stderr": "This is stderr",',
-                    '       "createdAt": "2021-10-05T12:34:56"',
+                    f'       "createdAt": "{test_go_timestamp}"',
+                    '     }',
+                    '  ]',
+                    '}',
+                ]) + '\n')
+
+            with open(test_path_file3, 'w') as f3:
+                f3.write('\n'.join([
+                    '{',
+                    '  "testCases": [',
+                    '     {',
+                    '       "testPathComponents": [',
+                    '         {',
+                    '           "type": "file",',
+                    '           "name": "test_java.py"',
+                    '         },',
+                    '         {',
+                    '           "type": "class",',
+                    '           "name": "class1"',
+                    '         },',
+                    '         {',
+                    '           "type": "testcase",',
+                    '           "name": "testcase#899"',
+                    '         }',
+                    '       ],',
+                    '       "duration": "34",',  # set duration as string
+                    '       "status": "TEST_PASSED",',
+                    '       "stdout": "This is stdout",',
+                    '       "stderr": "This is stderr",',
+                    f'       "createdAt": "{test_java_timestamp}"',
                     '     }',
                     '  ]',
                     '}',
@@ -172,43 +220,73 @@ class RawTest(CliTestCase):
             # emulate launchable record build
             write_build(self.build_name)
 
-            result = self.cli('record', 'tests', 'raw', test_path_file, test_path_file2, mix_stderr=False)
-            self.assertEqual(result.exit_code, 0)
+            result = self.cli('record', 'tests', 'raw', test_path_file, test_path_file2, test_path_file3, mix_stderr=False)
+            self.assert_success(result)
 
             # Check request body
-            payload = json.loads(gzip.decompress(responses.calls[2].request.body).decode())
+            payload = json.loads(gzip.decompress(self.find_request('/events').request.body).decode())
+            del payload['metadata']
             self.assert_json_orderless_equal(payload, {
                 'events': [
                     {
                         'testPath': [
-                            {'type': 'file', 'name': 'a.py'},
+                            {'type': 'file', 'name': 'test_ruby.py'},
                             {'type': 'class', 'name': 'classA'},
                         ],
                         'duration': 42,
                         'status': 1,
                         'stdout': 'This is stdout',
                         'stderr': 'This is stderr',
-                        'created_at': '2021-10-05T12:34:00',
+                        'createdAt': test_ruby_timestamp.replace(tzinfo=tzlocal()).isoformat(),
                         'data': None,
                         'type': 'case',
                     },
                     {
                         'testPath': [
-                            {'type': 'file', 'name': 'b.py'},
+                            {'type': 'file', 'name': 'test_go.py'},
                             {'type': 'class', 'name': 'classB'},
                         ],
                         'duration': 32,
                         'status': 1,
                         'stdout': 'This is stdout',
                         'stderr': 'This is stderr',
-                        'created_at': '2021-10-05T12:34:56',
+                        'createdAt': test_go_timestamp.replace(tzinfo=tzlocal()).isoformat(),
                         'data': None,
                         'type': 'case',
                     },
+                    {
+                        'testPath': [
+                            {'type': 'file', 'name': 'test_java.py'},
+                            {'type': 'class', 'name': 'class1'},
+                            {'type': 'testcase', 'name': 'testcase#899'},
+                        ],
+                        'duration': 34,
+                        'status': 1,
+                        'stdout': 'This is stdout',
+                        'stderr': 'This is stderr',
+                        'createdAt': test_java_timestamp.replace(tzinfo=tzlocal()).isoformat(),
+                        'data': None,
+                        'type': 'case',
+                    },
+                    {
+                        'testPath': [
+                            {'type': 'file', 'name': 'test_python.py'},
+                            {'type': 'class', 'name': 'classAA'},
+                        ],
+                        'duration': 12,
+                        'status': 1,
+                        'stdout': 'This is stdout',
+                        'stderr': 'This is stderr',
+                        'createdAt': test_python_timestamp.replace(tzinfo=tzlocal()).isoformat(),
+                        'data': {"lineNumber": 5},
+                        'type': 'case',
+                    },
                 ],
                 "testRunner": "raw",
                 "group": "",
                 "noBuild": False,
+                "flavors": [],
+                "testSuite": "",
             })
 
     @responses.activate
@@ -217,11 +295,14 @@ class RawTest(CliTestCase):
         with tempfile.TemporaryDirectory() as tempdir:
             test_path_file = os.path.join(tempdir, 'tests.xml')
             test_path_file2 = os.path.join(tempdir, 'tests_2.xml')
+            # These timestamp variables are defined for testing timezone.
+            timestamp1 = dateutil.parser.parse("2021-10-05T12:34:00")
+            timestamp2 = dateutil.parser.parse("2021-10-05T12:34:56")
             with open(test_path_file, 'w') as f:
                 f.write('\n'.join([
                     '<?xml version="1.0" encoding="UTF-8"?>',
                     '<testsuites name="test_suite_name1" tests="1" failures="0" errors="0" time="10.123">',
-                    '  <testsuite name="test_suite_name2" errors="0" failures="0" skipped="0" timestamp="2021-10-05T12:34:00" time="10.123" tests="1">',   # noqa: E501
+                    f'  <testsuite name="test_suite_name2" errors="0" failures="0" skipped="0" timestamp="{timestamp1}" time="10.123" tests="1">',   # noqa: E501
                     '    <testcase classname="test_class_name" name="test_case_name" time="10.123">',
                     '    </testcase>',
                     '  </testsuite>',
@@ -232,7 +313,7 @@ class RawTest(CliTestCase):
                 f2.write('\n'.join([
                     '<?xml version="1.0" encoding="UTF-8"?>',
                     '<testsuites name="test_suite_name3" tests="1" failures="0" errors="0" time="12.345">',
-                    '  <testsuite name="test_suite_name4" errors="0" failures="0" skipped="0" timestamp="2021-10-05T12:34:56" time="12.345" tests="1">',   # noqa: E501
+                    f'  <testsuite name="test_suite_name4" errors="0" failures="0" skipped="0" timestamp="{timestamp2}" time="12.345" tests="1">',   # noqa: E501
                     '    <testcase classname="test_class_name2" name="test_case_name2" time="12.345">',
                     '    </testcase>',
                     '  </testsuite>',
@@ -250,7 +331,8 @@ class RawTest(CliTestCase):
                     "Exit code is not 0. The output is\n" + result.output + "\n" + result.stderr)
 
             # Check request body
-            payload = json.loads(gzip.decompress(responses.calls[2].request.body).decode())
+            payload = json.loads(gzip.decompress(self.find_request('/events').request.body).decode())
+            del payload['metadata']
             self.assert_json_orderless_equal(payload, {
                 'events': [
                     {
@@ -262,7 +344,7 @@ class RawTest(CliTestCase):
                         'status': 1,
                         'stdout': '',
                         'stderr': '',
-                        'created_at': '2021-10-05T12:34:00',
+                        'createdAt': timestamp1.replace(tzinfo=tzlocal()).isoformat(),
                         'data': None,
                         'type': 'case',
                     },
@@ -275,14 +357,16 @@ class RawTest(CliTestCase):
                         'status': 1,
                         'stdout': '',
                         'stderr': '',
-                        'created_at': '2021-10-05T12:34:56',
+                        'createdAt': timestamp2.replace(tzinfo=tzlocal()).isoformat(),
                         'data': None,
                         'type': 'case',
                     },
                 ],
                 "testRunner": "raw",
                 "group": "",
-                "noBuild": False
+                "noBuild": False,
+                "flavors": [],
+                "testSuite": "",
             })
 
     @responses.activate
@@ -313,7 +397,8 @@ class RawTest(CliTestCase):
             rest.name,
             'raw')
 
-        self.assertEqual(result.exit_code, 0)
+        self.assert_success(result)
+
         self.assertEqual(
             result.stdout,
             '\n'.join([
