@@ -1,6 +1,6 @@
 from unittest import TestCase
 
-from click.testing import CliRunner
+from click.testing import CliRunner  # type: ignore
 
 from launchable.__main__ import main
 from launchable.version import __version__
@@ -11,5 +11,4 @@ class VersionTest(TestCase):
         runner = CliRunner()
         result = runner.invoke(main, ['--version'])
         self.assertEqual(result.exit_code, 0)
-        self.assertEqual(
-            result.output, 'launchable-cli, version {}\n'.format(__version__))
+        self.assertEqual(result.output, 'launchable-cli, version {}\n'.format(__version__))
